/* TIMER.C                                                                   */

/*---------------------------------------------------------------------------*/
/* Set up timers and suchlike                                                */

TmrInitialize() {

/*--- interrupt vectors                                                      */

 asm {
TMRHERE EQU *
   ORG   $000B                   /* Timer 0 interrupt     (IR pulses)        */
   LJMP  Tmr0Tick
   ORG   TMRHERE
 }

/*--- global variables                                                       */

   TmrBits = 0;                        /* flush all flags at once            */
   T0Ticks = 0;                        /* clear tick counter                 */

   T0Reload = T0RldNorm;               /* set up normal interrupt period     */

#ifndef POST_TEST                      /* we are using POST testing          */
   HBBits = HB_NORMAL;                 /* normal heartbeat pattern           */
#endif
   HBShift = 16;                       /* pattern is not shifted yet         */
   HBDelay = 1;                        /* force immediate update             */

/*--- Set up Timer 0 for long-period timings                                 */

   TCON &= 0xCF;                 /* clear run & interrupt flags              */

   TMOD &= 0xF0;                 /* set mode 1 (16-bit reload)               */
   TMOD |= 0x01;

   TH0 = TL0 = 0;                /* first delay is a long one                */

   SETBIT(ET0);                  /* enable the interrupt                     */
   SETBIT(TR0);                  /* start it running                         */

}


/*---------------------------------------------------------------------------*/
/* Timer 0 interrupt handler                                                 */
/* This is in assembler just to save some overhead                           */
/* The heartbeat code does not update the output to avoid clobbering any     */
/*  other I/O operations that may be in progress here...                     */
/*  so the heartbeat LED may stutter just a little bit                       */

Tmr0Tick() {

;
 asm {

   PUSH  A
   PUSH  PSW

*--- reload the timer

   CLR   TR0
   MOV   TL0,T0Reload
   MOV   TH0,T0Reload+1
   SETB  TR0

*--- update counter and check for a full second

   INC   TickCtr                       ; increment general timer
   INC   T0Ticks                       ; tick the wrap counter

   MOV   A,T0Ticks                     ; second yet?
   CJB   A,#TICKSPERSEC,T0Done

   SETB  FlagSecond                    ; yes, record this fact
   ADD   A,#-TICKSPERSEC               ; and knock off a full second
   MOV   T0Ticks,A

T0Done

*--- update heartbeat display

   DJNZ  HBDelay,T0Heart               ; time for another heartbeat output?
   MOV   HBDelay,#HEART_PERIOD         ; yes, reload delay counter

   MOV   A,HBBits+1                    ; fetch high bit for shift to low byte
   MOV   C,A.7
   MOV   A,HBBits                      ; low byte
   RLC   A
   MOV   HBBits,A
   MOV   A,HBBits+1                    ; high byte
   RLC   A
   MOV   HBBits+1,A

   MOV   FlagHeartBit,C                ; send data to internal LED flag

   DJNZ  HBShift,T0Flip                ; account for this shift
   MOV   HBShift,#16
   MOV   C,FlagHeart                   ; this should be clear by now
   ORL   C,FlagDead                    ; if it is not, we have up and died
   MOV   FlagDead,C

T0Flip
   CPL   FlagHeartBit                  ; flip data to be active low

T0Stall
   JB    FlagDead,T0Stall              ; stall if heartbeat code failure

   SETB  FlagHeart                     ; indicate we have wrapped the bits

T0Heart  EQU   *


*--- restore regs and return...

T0End    EQU   *
   POP   PSW
   POP   A


   RETI                          ; pre-empt normal C level return

 }

}


/*---------------------------------------------------------------------------*/
/* Check for heartbeat flag and clear it                                     */
/* If we fail to clear the flag by the next tick, we are doomed              */
/* Also update the output with the current heartbit bit                      */
/*  ... the interrupt routine cannot tweak the output directly               */

TmrHeartBeat() {


asm {

   CLR   FlagHeart                     ; turn this off to prevent lockup

   MOV   C,FlagHeartBit                ; internal flag is already inverted
   MOV   DIS_EEPROM,C
}

}


/*---------------------------------------------------------------------------*/
/* Flip current heartbeat output                                             */
/* This is intended for tight loops with interrupts off                      */

TmrFlipHeart() {

asm {
   MOV   C,HEARTBEAT
   CPL   C
   MOV   HEARTBEAT,C
}

}


/*---------------------------------------------------------------------------*/
/* Delay for a fixed number of milliseconds                                  */
/* Zero NumTicks will wait next tick                                         */

TmrDelayTicks(NumTicks)
unsigned NumTicks;
{
BYTE TickSample;

   do {
      TickSample = T0Ticks;            /* catch current timer tick counter   */
      while (TickSample == T0Ticks);   /* wait for timer tick                */
   } while (NumTicks--);               /* count each tick                    */



}


/*---------------------------------------------------------------------------*/
/* Delay for a fixed number of 100 microsecond ticks                         */
/* Zero NumUS is special-cased to bail out                                   */

TmrDelay100US(Num100US)
unsigned Num100US;
{

   if (!Num100US) {                    /* bail out for zero input            */
      return;
   }

   do {
 asm {
      MOV   B,#46
Tmr?1 EQU   *
      DJNZ  B,Tmr?1
 }
   } while (--Num100US);



}
