/* IRINTF.C -- IR transmitter/receiver interface                             */

/*---------------------------------------------------------------------------*/
/* Set up timers and suchlike                                                */

IRInitialize() {

/*--- interrupt vectors                                                      */

 asm {
IRHERE  EQU *
   ORG   $0003                   /* External Interrupt 0  (AC line sync)     */
   LJMP  ExtInt0
   ORG   $000B                   /* Timer 0 interrupt     (IR pulses)        */
   LJMP  Tmr0Tick
   ORG   IRHERE
 }

/*--- global variables                                                       */

/* CLRBIT(IRFLAG);  */                 /* no IR interrupt yet                */
   CLRBIT(ITRANS);                     /* shut off IR LED                    */

   IRBits0 = 0;                        /* flush all flags at once            */

   IRState = IR_IDLE;                  /* not doing anything just now        */
   IRecStat = 0;                       /* and not receiving anything         */
   T0Reload = T0RldNorm;               /* set up normal interrupt duration   */
   T0Ticks = 0;                        /* no counts so far                   */

   BadgeDly = 0;                       /* nothing sent so far...             */
   BadgeRsp = BADGE_DELAY;             /* and this is the default time       */

   memset(ITrnRing,0,sizeof(ITrnRing));            /* flush outgoing buffer  */
   IOutHead = IOutTail = IOutCtr = IOutMax = 0;    /* indicate no messages   */

   memset(IRecRing,0,sizeof(IRecRing));            /* flush incoming buffer  */
   IInHead = IInTail = IInCtr = IInMax = 0;        /* indicate no messages   */

   IRCtr = INumSend = 0;                           /* no bits so far         */
   memset(ITrnBuff,0,sizeof(ITrnBuff));            /* clear outgoing bits    */

   memset(IRMessages,0,sizeof(IRMessages));        /* nothing heard yet      */

/*--- Set up Timer 0 for long-period timings                                 */

   TCON &= 0xCF;                 /* clear run & interrupt flags              */

   TMOD &= 0xF0;                 /* set mode 1 (16-bit reload)               */
   TMOD |= 0x01;

   TH0 = TL0 = 0;                /* first delay is a long one                */

   SETBIT(PT0);                  /* high priority                            */
   SETBIT(ET0);
   SETBIT(TR0);                  /* start it running                         */

/*--- Set up External Interrupt 0 for edge-triggered mode                    */
/*    This must be low priority, as the timer controls output bit times      */

   SETBIT(IT0);
   SETBIT(EX0);

}


/*---------------------------------------------------------------------------*/
/* Timer 0 interrupt handler                                                 */
/* This measures out times for IR transmission & reception                   */
/* It's all in assembler because we need every cycle we can get...           */

Tmr0Tick() {

;
 asm {
*  SETB  TIMERFLAG                     ; *********
   PUSH  A
   PUSH  PSW

*--- reload the timer

   CLR   TR0
   MOV   TL0,T0Reload
   MOV   TH0,T0Reload+1
   SETB  TR0

*--- save IR sample in case we are receiving something

   MOV   C,FlagThisRBit                ; save previous sample for analysis
   MOV   FlagPrevRBit,C

   MOV   C,IRECEIVE                    ; get raw input bit from IR detector
   CPL   C                             ; fix active low inversion
   MOV   FlagThisRBit,C                ; save for later

*--- save bystanders for the rest of the interrupt

   PUSH  B
   PUSH  DPH
   PUSH  DPL
   PUSH  0                             ; R0 in disguise

*--- decide what we are doing based on the IRState control variable

   MOV   A,IRState                     ; fetch current state
   CJBE  A,#IR_RECEIVE,IRS_ok          ; ensure valid state
   MOV   A,#IR_IDLE                    ; oops...

IRS_ok   EQU   *
   MOV   B,#3                          ; get pointer to LJMPs
   MUL   AB
   MOV   DPTR,#T0TJmps                 ; dive into the table
   JMP   [A+DPTR]

T0TJmps  EQU   *
   LJMP  T0Normal
   LJMP  T0Trans
   LJMP  T0Rec

*--- normal timer tick, as there is no IR activity
*    check for transmitter data ready to go and switch states if so...
*    receiver interrupt handles its own setup without our attention

T0Normal EQU   *

   INC   T0Ticks                       ; record this interrupt

*--- update transmitter and polling delays
*    these only run when not transmitting or receiving to avoid speedup

   MOV   A,BadgeDly
   JZ    T0_n1                         ; stick at zero
   DEC   BadgeDly
T0_n1    EQU   *

   MOV   A,PollDly
   ORL   A,PollDly+1
   JZ    T0_n2                         ; stick at zero
   CLR   C
   MOV   A,PollDly
   SUBB  A,#1
   MOV   PollDly,A
   MOV   A,PollDly+1
   SUBB  A,#0
   MOV   PollDly+1,A
T0_n2    EQU   *

*--- now decide if there is something to transmit

   LJNB  FlagIRPending,T0Done          ; if no transmit ready, finished

   CLR   FlagIRPending                 ; indicate we saw the request
   MOV   IRState,#IR_TRANSMIT          ; set up state for next interrupt
   MOV   T0Ticks,#0                    ; nothing sent just yet
   MOV   IRCtr,#0                      ; no data out yet
   MOV   T0Reload,#LOW(T0RldTrans)     ; set up reload value
   MOV   T0Reload+1,#HIGH(T0RldTrans)
   CLR   TR0                           ; set up transmitter interrupt rate
   MOV   TL0,T0Reload
   MOV   TH0,T0Reload+1
   SETB  TR0

   CPL   MAINFLAG                      ; flag outgoing data

   LJMP  T0Done

*--- transmitting IR data

T0Trans  EQU   *

   MOV   DPTR,#ITrnBuff                ; get ptr to current bit
   MOV   A,IRCtr
   ADD   A,DPL
   MOV   DPL,A
   MOV   A,DPH
   ADDC  A,#0
   MOV   DPH,A
   MOVX  A,[DPTR]                      ; fetch the bit

   MOV   C,A.0                         ; use only LSB
***CPL   C                             ; output is low active
   MOV   ITRANS,C                      ; make it visible...

   INC   IRCtr                         ; get ready for next bit
   MOV   A,IRCtr

   LCJB  A,INumSend,T0Done             ; done with transmission?

   CLR   ITRANS                        ; force output OFF, just to be sure

   MOV   IRState,#IR_IDLE              ; yes, so return to idle state
   MOV   T0Reload,#LOW(T0RldNorm)      ; set up reload value
   MOV   T0Reload+1,#HIGH(T0RldNorm)
   CLR   TR0                           ; set up idle interrupt rate
   MOV   TL0,T0Reload
   MOV   TH0,T0Reload+1
   SETB  TR0

   MOV   A,INumSend                    ; add in the missing timer ticks
   MOV   B,#IRPERTICK                  ;  at this rate per interrupt
   DIV   AB
   ADD   A,T0Ticks
   MOV   T0Ticks,A

   MOV   DPTR,#BadgeRsp                ; force delay before next output
   MOVX  A,[DPTR]
   MOV   BadgeDly,A

   CPL   MAINFLAG                      ; indicate we are done now

   LJMP  T0Done



*--- receiving IR data
*    these ticks occur twice per bit frame, so we can verify each bit
*    if FlagSaveRBit is set, we skip the state to accumulate two samples

T0Rec    EQU   *

   LJBC  FlagSaveRBit,T0Done           ; skip and clear in one shot...

   MOV   A,IRecStat                    ; fetch current state
   CJBE  A,#RJUNK,IRD_ok               ; ensure valid state
   MOV   A,#RJUNK                      ; oops...
IRD_ok   EQU   *
   MOV   B,#3                          ; get 3-byte pointer
   MUL   AB                            ;  (LJMPs are three bytes each)
   MOV   DPTR,#T0RJmps                 ; dive into the table
   JMP   [A+DPTR]

T0RJmps  EQU   *
   LJMP  R1Sync1
   LJMP  R1Sync2
   LJMP  R1Data
   LJMP  R1Trail
   LJMP  R1Pause
   LJMP  R2Sync1
   LJMP  R2Sync2
   LJMP  R2Data
   LJMP  R2Trail
   LJMP  R2Pause
RJUNK EQU   *-T0RJmps/3                ; index for junk state
   LJMP  RxJunk


R1Sync1 EQU *                          ; just one sample for this bit frame
   JNB   FlagThisRBit,RCancel          ; if zero, a bad Int0 got us here...
   MOV   T0Reload,#LOW(T0RldRec)       ; set up normal reload value
   MOV   T0Reload+1,#HIGH(T0RldRec)
   CLR   TR0
   MOV   A,T0Reload                    ; tweak timer for current overhead
   ADD   A,#T0SyncAdj
   MOV   TL0,A
   CLR   A
   ADDC  A,T0Reload+1
   MOV   TH0,A
   SETB  TR0                           ; so next interrupt is on time
   INC   IRecStat
   LJMP  RSample

R1Sync2 EQU *                          ; second sync should be 0 bit
   MOV   C,FlagPrevRBit                ;  = 1?
   ANL   C,/FlagThisRBit               ;  = 0?
   JNC   RCancel                       ; 1 says both match up
   MOV   IRec1,#0                      ; so flush first bit accumulator
   MOV   IRec1+1,#0
   MOV   IRCtr,#NUMDATABITS            ; expect this many data bits
   INC   IRecStat
   SJMP  RSample

R1Data   EQU *
   XRLB  FlagPrevRBit,FlagThisRBit     ; must be different
   JNC   RCancel                       ; if not, bail out
   MOV   A,IRec1+1                     ; OK, so accumulate bits
   MOV   C,FlagThisRBit                ; second sample matches the logic value
   RRC   A
   MOV   IRec1+1,A
   MOV   A,IRec1                       ; do the low byte
   RRC   A
   MOV   IRec1,A                       ; data accumulates in high bits...

   DJNZ  IRCtr,RSample                 ; tick bit counter
   INC   IRecStat                      ; done, move to next state
   SJMP  RSample

R1Trail  EQU *                         ; trailer bit should be 0
   MOV   C,FlagPrevRBit                ;  = 1?
   ANL   C,/FlagThisRBit               ;  = 0?
   JNC   RCancel                       ; 1 says both match up
   MOV   IRCtr,#2                      ; so we expect two frames of pause
   INC   IRecStat
   SJMP  RSample

R1Pause  EQU *                         ; pause between repeats must be idle
   JB    FlagPrevRBit,RCancel          ;  ... so both samples are low
   JB    FlagThisRBit,RCancel          ;

   DJNZ  IRCtr,RSample                 ; tick bit counter
   INC   IRecStat                      ; done, move to next state
   SJMP  RSample

*--- utility routines to save some code...
*    these are in the middle to allow SJMPs from all over

RSample  EQU   *                       ; expect data bit next time...
   SETB  FlagSaveRBit                  ; get two samples for next state
   LJMP  T0Done

RCancel  EQU   *                       ; invalid format, so wait for pause
   MOV   IRecStat,#RJUNK
   LJMP  T0Done

RFinish  EQU   *
   MOV   IRState,#IR_IDLE
   MOV   IRecStat,#0
   MOV   T0Reload,#LOW(T0RldNorm)      ; timer to normal on next tick
   MOV   T0Reload+1,#HIGH(T0RldNorm)
   CPL   MAINFLAG                      ; indicate we are done now
   LJMP  T0Done

*--- continue with normal state routines

R2Sync1 EQU *
   MOV   C,FlagPrevRBit                ;  = 0?
   CPL   C                             ;  (no MOV C,/bit instruction...)
   ANL   C,FlagThisRBit                ;  = 1?
   JNC   RCancel                       ; 1 says both match up
   MOV   IRCtr,#NUMDATABITS            ; expect this many data bits
   INC   IRecStat
   SJMP  RSample

R2Sync2 EQU *
   MOV   C,FlagPrevRBit                ;  = 1?
   ANL   C,/FlagThisRBit               ;  = 0?
   JNC   RCancel                       ; 1 says both match up
   MOV   IRec2,#0                      ; so flush second bit accumulator
   MOV   IRec2+1,#0
   MOV   IRCtr,#NUMDATABITS            ; expect this many data bits
   INC   IRecStat
   SJMP  RSample

R2Data   EQU *
   XRLB  FlagPrevRBit,FlagThisRBit     ; must be different
   JNC   RCancel                       ; if not, bail out
   MOV   A,IRec2+1                     ; OK, so accumulate bits
   MOV   C,FlagThisRBit                ; second sample matches the logic value
   RRC   A
   MOV   IRec2+1,A
   MOV   A,IRec2                       ; do the low byte
   RRC   A
   MOV   IRec2,A                       ; data accumulates in high bits...

   DJNZ  IRCtr,RSample                 ; tick bit counter
   INC   IRecStat                      ; done, move to next state
   SJMP  RSample

R2Trail  EQU *
   MOV   C,FlagPrevRBit                ;  = 1?
   ANL   C,/FlagThisRBit               ;  = 0?
   JNC   RCancel                       ; 1 says both match up
   MOV   IRCtr,#3                      ; so expect 3 samples of pause
   INC   IRecStat
   SJMP  T0Done                        ; _must_ see next three samples here!

R2Pause  EQU *
   JB    FlagThisRBit,RCancel          ; sample must be inactive

   DJNZ  IRCtr,T0Done                  ; tick bit counter

*--- load results into the ring buffer here...
*    to get here we must have had a perfect message format...
*       but it is still worthwhile to compare the copies and scrap errors

   MOV   A,IRec1                       ; check low bytes
   XRL   A,IRec2
   JNZ   RFinish                       ; no match -> discard data

   MOV   A,IRec1+1                     ; check high bytes
   XRL   A,IRec2+1
   JNZ   RFinish                       ; no match -> discard data

   MOV   DPH,IRec1+1                   ; align data in low bits
   MOV   DPL,IRec1
   MOV   B,#16-NUMDATABITS
   CLR   C
T0_align EQU *
   MOV   A,DPH
   RRC   A
   MOV   DPH,A
   MOV   A,DPL
   RRC   A
   MOV   DPL,A
   DJNZ  B,T0_align
   MOV   IRec1+1,DPH
   MOV   IRec1,DPL

   MOV   A,IInHead                     ; get ring buffer pointer
   ADD   A,A                           ;  (which is word-sized)
   ADD   A,#LOW(IRecRing)
   MOV   DPL,A
   MOV   A,#HIGH(IRecRing)
   ADDC  A,#0
   MOV   DPH,A

   MOV   A,IRec1                       ; store data in ring
   MOVX  [DPTR],A
   INC   DPTR
   MOV   A,IRec1+1
   MOVX  [DPTR],A

   INC   IInHead                       ; increment the pointer
   MOV   A,#IRINLEN
   CJA   A,IInHead,T0_norr             ; and wrap if needed
   MOV   IInHead,#0
T0_norr  EQU   *

   INC   IInCtr                        ; update ring counter
   MOV   A,IInCtr
   CJBE  A,IInMax,T0_nom
   MOV   IInMax,A
T0_nom   EQU *

   MOV   A,#6                          ; adjust timer tick counter
   ADD   A,T0Ticks
   MOV   T0Ticks,A

   LJMP  RFinish                       ; successful completion!

*--- wait for a frame of no data, then bail out

RxJunk   EQU   *
   JB    FlagPrevRBit,T0Done           ;  = 0?
   JB    FlagThisRBit,T0Done           ;  = 0?
   LJMP  RFinish                       ; yes, so reset receiver

*--- have we gone a second since last interrupt?

T0Done   EQU   *

   MOV   A,T0Ticks                     ; fetch interrupt counter
   CJB   A,#TICKSPERSEC,T0End

   SETB  FlagSecond                    ; yes, record this fact
   ADD   A,#-TICKSPERSEC               ; and knock off a full second
   MOV   T0Ticks,A

*--- restore regs and return...

T0End    EQU   *
   POP   0                       ; restore R0
   POP   DPL
   POP   DPH
   POP   B
   POP   PSW
   POP   A

*  CLR   TIMERFLAG                     ; *********
   RETI                          ; pre-empt normal C level return

 }

}


/*---------------------------------------------------------------------------*/
/* External Interrupt 0 handler                                              */
/* This is triggered by high-to-low signal indicating IR energy from eyeball */
/* Manchester modulation means you do not get an interrupt for each bit!     */
/* We can only receive if the transmitter is idle                            */
/*  because we must synchronize the timer to the incoming bits...            */
/*  demodulation happens on the fly by running a separate state machine      */

ExtInt0() {

/*--- save bystanders                                                        */
;
 asm {
*  SETB  IRFLAG                        ; **********
   PUSH  A
   PUSH  B
   PUSH  PSW
   PUSH  DPH
   PUSH  DPL

   MOV   A,IRState                     ; make sure we can receive the bit
   CJBE  A,#IR_RECEIVE,IRR_ok          ; ensure valid state
   MOV   A,#IR_IDLE                    ; oops...
IRR_ok   EQU   *
   ADD   A,A
   MOV   DPTR,#RecJmp
   JMP   [A+DPTR]

RecJmp   EQU   *
   SJMP  RecNew
   SJMP  RecSkip
   SJMP  RecCont

*--- currently idle, so we can receive this one

RecNew   EQU   *

   MOV   T0Reload,#LOW(T0RldRecSync)   ; set up reload value for mid-bit sync
   MOV   T0Reload+1,#HIGH(T0RldRecSync)
   CLR   TR0                           ; punch it into the timer
   MOV   TL0,T0Reload
   MOV   TH0,T0Reload+1
   SETB  TR0

   MOV   IRecStat,#0                   ; starting new bit pattern now
   MOV   IRCtr,#0                      ; and no bit samples thus far
   MOV   IRState,#IR_RECEIVE           ; indicate the new state

   CPL   MAINFLAG                      ; indicate we are receiving now

   SJMP  I0Done

*--- currently transmitting, so we cannot jerk the timer around

RecSkip  EQU   *
   SJMP  I0Done

*--- reception in progress
*    this should occur when the timer is halfway to the next sample

RecCont  EQU   *

   CLR   TR0
   MOV   TL0,#LOW(T0RldRecAdj)         ; force correction for next bit frame
   MOV   TH0,#HIGH(T0RldRecAdj)
   SETB  TR0

   SJMP  I0Done

*--- restore bystanders and return

I0Done   EQU   *

   POP   DPL
   POP   DPH
   POP   PSW
   POP   B
   POP   A
*  CLR   IRFLAG                        ; **********
   RETI                          ; pre-empt normal return
 }

}


/*---------------------------------------------------------------------------*/
/* Convert argument into Manchester data frame (two outputs per call)        */
/* Uses only low bit of argument to simplify the calls                       */
/* Output is 1 for "IR On" and 0 for "IR Off"                                */

IRFormatFrame(MsgBit,pMsgFrame)
unsigned int MsgBit;
BYTE *pMsgFrame;
{

   if (0 == (MsgBit & 0x0001)) {
      *pMsgFrame = 1;
      *(pMsgFrame+1) = 0;
   }
   else {
      *pMsgFrame = 0;
      *(pMsgFrame+1) = 1;
   }


}


/*---------------------------------------------------------------------------*/
/* Convert argument into Manchester-encoded burst at ptr                     */

IRFormatBurst(MsgNum,pMsgBase)
unsigned int MsgNum;
BYTE *pMsgBase;
{

BYTE Index;

   IRFormatFrame(1,pMsgBase+0);        /* start bits                         */
   IRFormatFrame(0,pMsgBase+2);
   pMsgBase += 4;

   for (Index=0; Index<NUMDATABITS; ++Index) {     /* encode message         */
      IRFormatFrame(MsgNum,pMsgBase);
      MsgNum >>= 1;
      pMsgBase += 2;
   }

   IRFormatFrame(0,pMsgBase);          /* trailing bit                       */

}


/*---------------------------------------------------------------------------*/
/* Convert integer into bit pattern in the transmitter buffer                */
/* This uses MC145030 Manchester encoded format with repeated bursts         */
/* Sets flag to start transmission when buffer is ready                      */
/* Assumes that the previous transmission is finished...                     */
/*  which means you had best check FlagIRPending before calling here...      */

IRFormatMsg(MsgNum)
WORD MsgNum;
{

   if ((0xFFFF != MsgNum) && (REPORT_TRANS & ReportMode)) {
      sprintf(CmdRespBuff,"Trn %u [%04x hex]\n",MsgNum,MsgNum);
      Nputstr(CmdRespBuff);
   }

   memset(ITrnBuff,0,sizeof(ITrnBuff));            /* flush previous bits    */

   IRFormatBurst(MsgNum,ITrnBuff+24);              /* first transmission     */
   IRFormatBurst(MsgNum,ITrnBuff+52);              /* repeat for emphasis    */

   INumSend = sizeof(ITrnBuff);        /* how many half-frames in the buffer */

   SETBIT(FlagIRPending);              /* indicate ready to go               */
}


/*---------------------------------------------------------------------------*/
/* Return most recent entry from the IR receiver ring buffer                 */
/* If nothing, you get 0xFFFF                                                */

WORD IRGetInRing() {

WORD MsgNum;

   if (IInCtr) {
      MsgNum = IRecRing[IInTail];
 asm {
      DEC   IInCtr                     ; must be atomic operation!
 }
      ++IInTail;                       /* tick and wrap index                */
      if (IRINLEN <= IInTail) {
         IInTail = 0;
      }
   }
   else {
      MsgNum = 0xFFFF;
   }

/*--- dump result if we are logging receiver successes                       */

   if ((0xFFFF != MsgNum) && (REPORT_REC & ReportMode)) {
         sprintf(CmdRespBuff,"Rcv %u [%04x hex]\n",MsgNum,MsgNum);
         Nputstr(CmdRespBuff);
   }

   return MsgNum;

}


/*---------------------------------------------------------------------------*/
/* Check for anything in the receiver and update the "heard from" table      */

IRCheckRec() {

WORD RecValue;

   RecValue = IRGetInRing();

   if ((0xFFFF != RecValue) && (RecValue < MAXMSGNUM)) {
      IRMessages[RecValue] = 1;
   }


}



/*---------------------------------------------------------------------------*/
/* Add entry to IR transmitter ring buffer                                   */
/* Returns 0 if sent, 1 if not sent                                          */

int IRSendMsg(MsgVal)
WORD MsgVal;
{

int RetVal;

   if ((MsgVal <= MAXMSGNUM) && (IOutCtr < IROUTLEN)) {
      ITrnRing[IOutHead] = MsgVal;
      ++IOutHead;
      if (IROUTLEN <= IOutHead) {
         IOutHead = 0;
      }
      ++IOutCtr;
      if (IOutCtr > IOutMax) {
         IOutMax = IOutCtr;
      }
      RetVal = 0;
   }
   else {
      RetVal = 1;
   }

   return RetVal;

}

/*---------------------------------------------------------------------------*/
/* Return oldest entry from the IR transmitter ring buffer                   */
/* If nothing, you get 0xFFFF                                                */

WORD IRGetOutRing() {

WORD RetVal;

   if (IOutCtr) {
      RetVal = ITrnRing[IOutTail];
 asm {
      DEC   IOutCtr                    ; must be atomic operation!
 }
      ++IOutTail;                       /* tick and wrap index                */
      if (IROUTLEN <= IOutTail) {
         IOutTail = 0;
      }
   }
   else {
      RetVal = 0xFFFF;
   }

   return RetVal;

}


/*---------------------------------------------------------------------------*/
/* Check to see if there is anything to send                                 */
/* If so, make sure that the transmitter has been idle long enough to allow  */
/*  a response from a badge listening to our output...                       */
/* and make sure we are not transmitting anything right now, either...       */

IRCheckTrans() {

WORD MsgVal;

   if ((!BadgeDly) && (IR_IDLE == IRState)) {      /* allowed to send now    */
      MsgVal = IRGetOutRing();
      if (0xFFFF != MsgVal){                       /* anything to send?      */
         IRFormatMsg(MsgVal);                      /* do it...               */
      }
      BadgeDly = BadgeRsp;                         /* reset delay to next    */
   }

}


/*---------------------------------------------------------------------------*/
/* Dump IR message response table                                            */
/* All dumped values are cleared to ensure you do not see them again...      */
/* Note that the format changes depending on how you ask the question        */

IRDumpMsg(First,Last)
int First;
int Last;
{
BYTE Aggregate;
int Index;

   if (First == Last) {                /* single entry, special format       */
      sprintf(CmdRespBuff," %03u=%u\n",
         First, IRMessages[First] ? 1 : 0);
      Nputstr(CmdRespBuff);
      IRMessages[First] = 0;
   }
   else {                              /* multiple entries, do a line        */
      sprintf(CmdRespBuff," %03u-%03u=",First,Last);
      Nputstr(CmdRespBuff);
      while (First <= Last) {
         Aggregate = 0;
         for (Index=First;
              (Index < (First+8)) && (Index <= Last);
              ++Index) {
            Aggregate >>= 1;
            Aggregate |= IRMessages[Index] ? 0x80 : 0;
            IRMessages[Index] = 0;
         }
         Aggregate >>= (First+8) - Index;          /* align to low bits      */
         sprintf(CmdRespBuff,"%02x",Aggregate);
         Nputstr(CmdRespBuff);
         First += 8;
      }
      Nputch('\n');
   }

}



/*---------------------------------------------------------------------------*/
/* Dump current IR state                                                     */
/* Side effect resets the maximum values of the ring buffer counters         */

IRDumpState() {

   Nputstr("IR interface status:\n");

   sprintf(CmdRespBuff,
          " IRState %02x, IRecStat %02x, BadgeDly %u, BadgeRsp %u\n",
          IRState,IRecStat,BadgeDly,BadgeRsp);
   Nputstr(CmdRespBuff);

   sprintf(CmdRespBuff," IRCtr %02x, ",
          IRCtr);
   Nputstr(CmdRespBuff);
   sprintf(CmdRespBuff,"INumSend %02x, IRBits0 %02x, T0Ticks %02x\n",
            INumSend,IRBits0,T0Ticks);
   Nputstr(CmdRespBuff);

   sprintf(CmdRespBuff,
          " Trans:  Head %02x, Tail %02x, Ctr %02x, Max %02x, ",
          IOutHead,IOutTail,IOutCtr,IOutMax);
   Nputstr(CmdRespBuff);
   sprintf(CmdRespBuff,"[IOutHead]->%04x, ",ITrnRing[IOutHead]);
   Nputstr(CmdRespBuff);
   sprintf(CmdRespBuff,"[IOutTail]->%04x\n",ITrnRing[IOutTail]);
   Nputstr(CmdRespBuff);

   sprintf(CmdRespBuff,
          " Rec:    Head %02x, Tail %02x, Ctr %02x, Max %02x, ",
          IInHead,IInTail,IInCtr,IInMax);
   Nputstr(CmdRespBuff);
   sprintf(CmdRespBuff,"[IInHead]-> %04x, ",IRecRing[IInHead]);
   Nputstr(CmdRespBuff);
   sprintf(CmdRespBuff,"[IInTail]-> %04x",IRecRing[IInTail]);
   Nputstr(CmdRespBuff);
   Nputch('\n');


   IInMax = IOutMax = 0;

}

/*---------------------------------------------------------------------------*/
/* Count one second of T1 input and return as unsigned int                   */
/* Must run with interrupts off to ensure good timing                        */
/* At each timer tick we flip loops because they be so long...               */

#define T0FREQLOAD -46080              /* 50 ms almost exactly               */
#define FREQLOOPS 20                   /* iterations to make one second      */

unsigned int IRCount() {


 asm {

   MOV   DPTR,#0                       ; clear cycle counter
   MOV   B,#FREQLOOPS                  ; set up loop counter

   CLR   EA                            ; ensure no distractions
   CLR   MAINFLAG                      ; indicate we are waiting

   CLR   TR0                           ; set up timer for first loop
   CLR   TF0
   MOV   TH0,#HIGH(T0FREQLOAD)
   MOV   TL0,#LOW(T0FREQLOAD)
   SETB  TR0                           ; start counting

IRC?1    EQU   *
   JBC   TF0,IRC?3                     ; 2 timer wrapped?
   JNB   IRFREQMON,IRC?1               ; 2 wait for 1 input bit

IRC?2    EQU   *
   JBC   TF0,IRC?4                     ; 2 timer wrapped?
   JB    IRFREQMON,IRC?2               ; 2 wait for 0 input bit

   INC   DPTR                          ; 2 record the edge
   SJMP  IRC?1                         ; 2 wait for next 1 input bit

IRC?3    EQU   *                       ; waiting for 1 bit
   MOV   TH0,#HIGH(T0FREQLOAD)         ; 2 TL0 just wrapped, OK to load
   MOV   TL0,#LOW(T0FREQLOAD)          ; 2 and this resets TL0
   CPL   MAINFLAG                      ; 2 flag it
   JB    RI,IRC_done                   ; 2 check for keypress
   DJNZ  B,IRC?2                       ; 2 return to _0_ loop!!
   SJMP  IRC_done                      ; which just happened

IRC?4    EQU   *                       ; waiting for 0 bit
   MOV   TH0,#HIGH(T0FREQLOAD)         ; 2 TL0 just wrapped, OK to load
   MOV   TL0,#LOW(T0FREQLOAD)          ; 2 and this resets TL0
   CPL   MAINFLAG                      ; 2 flag it
   JB    RI,IRC_done                   ; 2 check for keypress
   DJNZ  B,IRC?1                       ; 2 return to _1_ loop!!

IRC_done EQU   *

*--- set up return variable

   MOV   B,DPH
   MOV   A,DPL
   MOV   DPTR,#IRFreq
   MOVX  [DPTR],A
   INC   DPTR
   MOV   A,B
   MOVX  [DPTR],A

   SETB  MAINFLAG                      ; restore normal output
   SETB  TF0                           ; force normal timer tick
   SETB  EA                            ; and here we go...

 }

   return IRFreq;

}



/*---------------------------------------------------------------------------*/
/* Show on-board oscillator frequency                                        */
/* This runs with interrupts off to ensure good timing...                    */

#define IRTC_MID     38000
#define IRTC_RANGE    4000
#define IRTC_INCR       40

#define IRTC_MIN     (IRTC_MID-IRTC_RANGE/2)
#define IRTC_MAX     (IRTC_MID+IRTC_RANGE/2)
#define IRTC_STEP    (IRTC_RANGE/IRTC_INCR)

IRCalTrans(ShowDetail)
int ShowDetail;
{

unsigned int NumCycles;
unsigned int CycScan;

   ShowDetail = ShowDetail;                        /* unused for now         */

   Nputstr("--- Connect U11.1 to T1 input\n");
   sprintf(CmdRespBuff,"     Center %u Hz, range %u Hz, step  %u Hz\n",
           IRTC_MID,IRTC_RANGE,IRTC_STEP);
   Nputstr(CmdRespBuff);
   Nputstr("    Press any key to quit...\n");

   do {
      SerWaitOutDone();
      NumCycles = IRCount();

      if (chkch()) {
         break;                        /* skip displays when cancelled       */
      }

      sprintf(CmdRespBuff,"%6u Hz [",NumCycles);
      Nputstr(CmdRespBuff);

      if (NumCycles < IRTC_MIN) {
         sprintf(CmdRespBuff,"<<< under %u]\n",IRTC_MIN);
         Nputstr(CmdRespBuff);
         continue;
      }

      if (NumCycles > IRTC_MAX) {
         sprintf(CmdRespBuff,"over %u >>>]\n",IRTC_MAX);
         Nputstr(CmdRespBuff);
         continue;
      }

      for (CycScan  = (IRTC_MIN-IRTC_STEP/2);
           CycScan <= (IRTC_MAX+IRTC_STEP/2);
           CycScan += IRTC_STEP) {
         if ((NumCycles >= CycScan) && (NumCycles <= (CycScan+IRTC_STEP))) {
            Nputch('*');                           /* doubled on transitions */
         }
         else {
            if ((IRTC_MID-IRTC_STEP/2) == CycScan) {
               Nputch('|');
            }
            else {
               Nputch(' ');
            }
         }
      }
      Nputstr("]\n");

   } while (!chkch());                             /* assume interrupts      */

   SerFlushIn();                                   /* discard inputs         */

   Nputstr("--- Disconnect U11.1 from T1 input\n");

}



/*---------------------------------------------------------------------------*/
/* Sample incoming (demodulated) IR bit stream & record times                */
/* This is done without interrupts to get best timing accuracy               */
/* We do not see the IR carrier signal through the Sharp receiver, though    */
/* Measure total ON time for first pulse to get nominal frame time           */

IRSample() {

 asm {

   MOV   DPTR,#IRTimes                 ; set timer record array
   MOV   R1,#MAXIRFRAMES-1             ; set up frame counter

   CLR   EA                            ; ensure no distractions
   CLR   MAINFLAG                      ; indicate we are waiting

   CLR   TR0                           ; set up timer to show blinking
   CLR   TF0
   MOV   TH0,#0
   MOV   TL0,#0
   SETB  TR0

IRS?1    EQU   *
   JNB   IRECEIVE,IRS?2                ; first incoming bit arrived?
   JNB   TF0,IRS?1                     ; timer wrapped?
   CLR   TF0                           ; yes, so...
   CPL   MAINFLAG                      ;   flip blinking bit
   JB    RI,IRS_done                   ; check for keypress
   SJMP  IRS?1                         ; keep waiting for first bit

IRS?2    EQU   *                       ; got a bit, so start real wait...
   CLR   TR0
   MOV   TH0,#0                        ; set message timeout
   MOV   TL0,#0                        ;  ...
   SETB  TR0
   CLR   TF0

IRS?3    EQU   *                       ; sample first ON time
   JB    TF0,IRS_done                  ; until timeout...
   JNB   IRECEIVE,IRS?3                ; while input remains active

IRS?3a
   MOV   A,TH0                         ; snag the timer value
   MOV   B,TL0
   CJNE  A,TH0,IRS?3a                  ; (redo if we hit a carry...)

   CJBE  A,#3,IRS?1                    ; less than 1024 counts is a glitch
   XCH   A,B

   MOVX  [DPTR],A                      ; and record it in the array
   INC   DPTR
   MOV   A,B
   MOVX  [DPTR],A
   INC   DPTR

IRS?5    EQU   *                       ; now time each bit until...
   JB    TF0,IRS_done                  ;  ... timeout
   JB    IRECEIVE,IRS?5                ; wait for bit active

IRS?5a
   MOV   A,TH0                         ; snag the timer value
   MOV   B,TL0
   CJNE  A,TH0,IRS?5a                  ; (redo if we hit a carry...)
   XCH   A,B

   MOVX  [DPTR],A                      ; and record it in the array
   INC   DPTR
   MOV   A,B
   MOVX  [DPTR],A
   INC   DPTR

IRS?6    EQU   *                       ; wait for bit inactive
   JB    TF0,IRS_done                  ;  ... or timeout
   JNB   IRECEIVE,IRS?6

   DJNZ  R1,IRS?5                      ; if more frames, continue

IRS_done EQU   *

   SETB  MAINFLAG                      ; clear output flag
   SETB  EA                            ; allow immediate timer interrupt

 }

}

/*---------------------------------------------------------------------------*/
/* Show remote MC145030 oscillator params                                    */
/* Derives modulation frequency by measuring bit frame times...              */

IRCalRemote(ShowDetail)
int ShowDetail;
{

BYTE FrameNum;
BYTE FrameCount;
BYTE Index;
unsigned int FrameTime;
unsigned int FrameTime1;
unsigned int FrameTime2;
unsigned int AvgFrTime;

   Nputstr("--- Trigger remote unit for each measurement\n");
   Nputstr("     MC145030 clock is 16/(bit frame time)\n");
   Nputstr("    Press any key to quit...\n");

   do {
      memset(IRTimes,0,sizeof(IRTimes));
      SerWaitOutDone();
      IRSample();

      if (chkch() && !IRTimes[0]) {
         break;                        /* skip all the display if cancelled  */
      }

/*--- get differential time between each sample                              */
/*    [0] is the start bit frame time, which estimates the true frame time   */
/*    [1] is time since the start, so it is already a delta time             */

      for (FrameNum = MAXIRFRAMES-1; FrameNum > 1; --FrameNum) {
         IRTimes[FrameNum] = IRTimes[FrameNum] - IRTimes[FrameNum-1];
      }

/*--- convert times from timer ticks to microseconds                         */
/*    the magic number is 1.085 microseconds per timer tick, but             */
/*     we must do it in two tiny chunks to avoid overflowing 16 bits         */

      for (FrameNum = 0; FrameNum < MAXIRFRAMES; ++FrameNum) {
         if (ShowDetail >= 3) {
            sprintf(CmdRespBuff," Sample %2u raw=%04x=%5u ",
                     FrameNum,IRTimes[FrameNum],IRTimes[FrameNum]);
            Nputstr(CmdRespBuff);
         }

         FrameTime1 = IRTimes[FrameNum] * 2;                /* 0.08          */
         FrameTime1 = (FrameTime1 + 12) / 25;
         FrameTime2 = (IRTimes[FrameNum] + 100)/200;        /* 0.005         */

         IRTimes[FrameNum] += FrameTime1 + FrameTime2;      /* 1.085         */
         if (ShowDetail >= 3) {
            sprintf(CmdRespBuff,"cvt=%04x=%5u us\n",
                     IRTimes[FrameNum],IRTimes[FrameNum]);
            Nputstr(CmdRespBuff);
         }
      }

/*--- set up initial frame sorting thresholds                                */
/*    this is done in chunks to avoid overflows, too                         */
/*    first frame is special-cased sample, needs 80% derating to real time   */
/*     because the trailing edge is delayed by IR receiver                   */

      FrameTime1 = IRTimes[0] * 8;
      FrameTime1 = FrameTime1 / 10;
      if (ShowDetail >= 1) {
         sprintf(CmdRespBuff," Est bit frame adjusted from %u to %u us\n",
                  IRTimes[0],FrameTime1);
         Nputstr(CmdRespBuff);
         Nputstr(" Bit frame time thresholds: ");
      }
      IRTimes[0] = FrameTime1;
      for (Index = 0; Index < NUMTHRESH; ++Index) {
         IRThresh[Index] = IRTimes[0] * (2*(Index+1) + 1);
         IRThresh[Index] >>= 2;
         if (ShowDetail >= 1) {
            sprintf(CmdRespBuff,"%u ",IRThresh[Index]);
            Nputstr(CmdRespBuff);
         }
      }
      if (ShowDetail >= 1) {
         Nputstr("us\n");
      }

/*--- now find the average frame time                                        */
/*    determine each interrupt duration (1.0, 1.5, 2.0... bit frame times)   */

      AvgFrTime = 0;
      FrameCount = 0;
      for (FrameNum = 1;
           (FrameNum < MAXIRFRAMES) && (IRTimes[FrameNum]);
           ++FrameNum) {
         if (ShowDetail >= 2) {
            sprintf(CmdRespBuff," Sample %2u %5u us",
                        FrameNum,IRTimes[FrameNum],IRThresh[0]);
            Nputstr(CmdRespBuff);
         }
         if (IRTimes[FrameNum] <= IRThresh[0]) {
            if (IRTimes[FrameNum]) {
               if (ShowDetail >= 2) {
                  Nputstr(", glitch ignored\n");
               }
            }
            else {
               if (ShowDetail >= 2) {
                  Nputstr(", unused\n");
               }
            }
            continue;
         }
         else {
            if (IRTimes[FrameNum] > IRThresh[NUMTHRESH-1]) {
               if (ShowDetail >= 2) {
                  Nputstr(", long space ignored\n");
               }
               continue;
            }
         }

         for (Index = 1; Index < (NUMTHRESH-1); ++Index) {
            if (IRTimes[FrameNum] <= IRThresh[Index]) {
               FrameTime = IRTimes[FrameNum] * 2;
               FrameTime /= Index + 1;
               if (ShowDetail >= 2) {
                  sprintf(CmdRespBuff,", %u half-frames -> %u us/frame ",
                              Index+1,FrameTime);
                  Nputstr(CmdRespBuff);
               }
               AvgFrTime += FrameTime;
               ++FrameCount;
               break;
            }
         }

         if (ShowDetail >= 2) {
            Nputch('\n');
         }
      }

/*--- got the answer at last!                                                */
/*    clock frequency is 16 times the frame frequency                        */

      AvgFrTime /= FrameCount;

#ifdef x0
      BitClock = (AvgFrTime+5) / 10;
      BitClock = 16000 / BitClock;
      BitClock = BitClock * 100;

      sprintf(CmdRespBuff," Bit frame time is %u us, bit clock %u Hz\n",
                  AvgFrTime,BitClock);
#else
      sprintf(CmdRespBuff," Bit frame time is %u us\n",AvgFrTime);
#endif

      Nputstr(CmdRespBuff);

   } while (!chkch());                             /* assume interrupts      */

   SerFlushIn();                                   /* discard inputs         */


}



