/* MCIR.C -- Master Controller IR interface                                  */

/*---------------------------------------------------------------------------*/
/* Set up new RAM data structure                                             */

MCInitData() {

   pMCData = pMCRAM;

   memset(pMCData,0,MCRAMSize);
   MCRecs = 0;
   MCPulses = 0;
   pMCThis = pMCData + 2;
   pMCNext = pMCThis;
   MCRecID = 0;

}

/*---------------------------------------------------------------------------*/
/* Initialize MCIR interface                                                 */

MCInitialize() {

/*--- set up global variables                                                */

   MCBits0 = 0 ;                       /* clear all the flags                */

   MCDuty = FIX_DUTY;                  /* set up duty cycle adjustment       */

/*--- see if the RAM data area needs to be set up                            */

   pMCData = pMCRAM;

   if (MCFetchRec(MAXRECNUM) < 0) {
      CmdShowMsg("No data in RAM, initializing empty structure\n");
      MCInitData();
   }

   return;

}


/*---------------------------------------------------------------------------*/
/* Find and decompress an IR record                                          */
/* pMCData must point to the start of the IR data block                      */
/* Sets up global variables with number of pulses in record:                 */
/*    0 if record not found or bogus input ID                                */
/*   -1 if data structure is invalid                                         */

int MCFetchRec(SeekID)
int SeekID;
{

   SeekID = SeekID;                    /* keep compiler happy                */

   if (REPORT_DECOMP & ReportMode) {
      sprintf(CmdRespBuff,"IR data at %04x, seeking record %u\n",
         pMCData,SeekID);
      Nputstr(CmdRespBuff);
   }

   MCPulses = 0;                       /* assume we will not find record     */

   if (!(*(WORD *)pMCData)) {          /* bail out if no pulses              */
      ErrFlags |= ERR_SBAD;
      MCPulses = -1;
      goto Done;
   }

   if ((MAXRECNUM < *(WORD *)pMCData) || (! *(WORD *)pMCData)) {
      ErrFlags |= ERR_SBAD;            /* record number is no good           */
      MCPulses = -1;
      goto Done;
   }

   if (REPORT_DECOMP & ReportMode) {
      Nputstr("--- Dump before decompression\n");
      MCDumpState();
   }

asm {

   MOV   DPTR,#pMCData                 ; get pointer to compressed data addr
   MOVX  A,[DPTR]                      ; low address
   INC   DPTR
   MOV   B,A
   MOVX  A,[DPTR]                      ; high address

   MOV   DPH,A                         ; set up data address
   MOV   DPL,B

   MOVX  A,[DPTR]                      ; pick up total record count
   INC   DPTR
   MOV   R2,A                          ;  low byte
   MOVX  A,[DPTR]
   INC   DPTR
   MOV   R3,A                          ;  high byte

   PUSH  DPL                           ; store for later
   PUSH  DPH
   MOV   DPTR,#MCRecs
   MOV   A,R2
   MOVX  [DPTR],A
   INC   DPTR
   MOV   A,R3
   MOVX  [DPTR],A
   POP   DPH
   POP   DPL

*--- unpack each record until we match the ID number
*     or we hit the end of the data
*     or we exceed the record count limit

?GetRec  EQU *

   JNZRR R2,R3,?GR0                    ; are we out of records yet?
   ORL   ErrFlags,#ERR_NBAD
   MOV   DPTR,#MCPulses                ; indicate not found
   CLR   A FF
   MOVX  [DPTR],A
   INC   DPTR
   MOVX  [DPTR],A
   LJMP  ?Quit                         ; count zero, so we quit

?GR0

   MOV   A,DPL                         ; save pointer to start address
   MOV   B,DPH
   PUSH  DPL
   PUSH  DPH
   MOV   DPTR,#pMCThis
   MOVX  [DPTR],A
   INC   DPTR
   MOV   A,B
   MOVX  [DPTR],A
   POP   DPH
   POP   DPL

   MOVX  A,[DPTR]                      ; get record ID number from data
   INC   DPTR
   MOV   MCRecID,A                     ;  low byte
   MOVX  A,[DPTR]
   INC   DPTR
   MOV   MCRecID+1,A                   ;  high byte

*--- define decompression states

UNP_WDS  SET   0                       ; unpack times stored in words
UNP_BYTS SET   1                       ; unpack times stored in bytes
UNP_NOWD SET   2                       ; word count was zero
UNP_NOBY SET   3                       ; byte count was zero
UNP_DONE SET   4                       ; finished!

*--- step through current record, decompressing blocks as we go

?RunRec  EQU *

   MOVX  A,[DPTR]                      ; get pulse count for this record
   INC   DPTR
   MOV   R6,A                          ; low byte
   MOVX  A,[DPTR]
   INC   DPTR
   MOV   R7,A

   PUSH  DPL
   PUSH  DPH
   MOV   DPTR,#MCPulses                ; store in global variable
   MOV   A,R6
   MOVX  [DPTR],A
   INC   DPTR
   MOV   A,R7
   MOVX  [DPTR],A
   POP   DPH
   POP   DPL

   MOV   MCAddr,#HIGH(MCTimes)         ; aim at decompressed buffer
   MOV   R1,#LOW(MCTimes)

   MOV   R4,#UNP_WDS                   : first block is word pulse
   MOV   R5,#1                         ; with one pulse therein...

*--- unpack pulses within this record

?UnPack  EQU *

   CPL   HEARTBEAT                     ; indicate we are alive...

*--- update the total number of pulses to reflect what is about to happen
*    the count may go to zero, but not below...

   MOV   A,R6                          ; subtract pulses from total count
   CLR   C
   SUBB  A,R5
   MOV   R6,A
   MOV   A,R7
   SUBB  A,#0
   MOV   R7,A
   LJC   ?badx                         ; carry (borrow) says bad count!

*---- blocks with two bytes per time, four bytes per pulse

   CJNE  R4,#UNP_WDS,?unp1

   MOV   A,R5                          ; convert pulse count to time slots
   MOV   B,#2
   MUL   AB
   MOV   R4,A                          ; low byte
   MOV   R5,B                          ; high byte

?unp00
   MOVX  A,[DPTR]                      ; move two bytes directly
   INC   DPTR
   MOV   P2,MCAddr
   MOVX  [R1],A
   INC   R1
   CJNE  R1,#0,?unp01
   INC   MCAddr
?unp01
   MOVX  A,[DPTR]
   INC   DPTR
   MOV   P2,MCAddr
   MOVX  [R1],A
   INC   R1
   CJNE  R1,#0,?unp02
   INC   MCAddr
?unp02
   CLR   A                             ; fill in high byte with zero
   MOV   P2,MCAddr
   MOVX  [R1],A
   INC   R1
   CJNE  R1,#0,?unp03
   INC   MCAddr
?unp03
   DecRR R5,R4                         ; tick counter
   JNZRR R4,R5,?unp00

   MOV   R4,#UNP_BYTS                  ; assume next block is bytes
   MOVX  A,[DPTR]                      ; get next rep count
   INC   DPTR
   MOV   R5,A
   JNZ   ?EndLp                        ; if zero, signal "no bytes" state
   MOV   R4,#UNP_NOBY
   SJMP  ?EndLp

*--- block with one byte per time, two bytes per pulse

?unp1    EQU *
   CJNE  R4,#UNP_BYTS,?unp2

   MOV   A,R5                          ; convert pulse count to times
   MOV   B,#2
   MUL   AB
   MOV   R4,A                          ; low byte
   MOV   R5,B                          ; high byte

?unp10
   MOVX  A,[DPTR]                      ; move timing byte
   INC   DPTR
   MOV   P2,MCAddr
   MOVX  [R1],A
   INC   R1
   CJNE  R1,#0,?unp11
   INC   MCAddr
?unp11
   CLR   A                             ; fill in zero high bytes
   MOV   P2,MCAddr
   MOVX  [R1],A
   INC   R1
   CJNE  R1,#0,?unp12
   INC   MCAddr
?unp12
   MOV   P2,MCAddr
   MOVX  [R1],A
   INC   R1
   CJNE  R1,#0,?unp13
   INC   MCAddr
?unp13
   DecRR R5,R4                         ; tick counter
   JNZRR R4,R5,?unp10

   MOV   R4,#UNP_WDS                   ; assume next block is words
   MOVX  A,[DPTR]                      ; get next rep count
   INC   DPTR
   MOV   R5,A
   JNZ   ?EndLp                        ; if zero, signal "no words" state
   MOV   R4,#UNP_NOWD
   SJMP  ?EndLp

*--- previous word block was empty, if this count is zero we are done

?unp2    EQU *
   CJNE  R4,#UNP_NOWD,?unp3

   MOV   R4,#UNP_BYTS                  ; assume next block is bytes
   MOVX  A,[DPTR]                      ; get next rep count
   INC   DPTR
   MOV   R5,A
   JNZ   ?EndLp                        ; if zero, signal "done" state
   MOV   R4,#UNP_DONE
   SJMP  ?EndLp

*--- previous byte block was empty, if this count is zero we are done

?unp3    EQU *
   CJNE  R4,#UNP_NOBY,?badx

   MOV   R4,#UNP_WDS                   ; assume next block is words
   MOVX  A,[DPTR]                      ; get next rep count
   INC   DPTR
   MOV   R5,A
   JNZ   ?EndLp                        ; if zero, signal "done" state
   MOV   R4,#UNP_DONE
   SJMP  ?EndLp

*--- invalid state or error in other sections...

?badx    EQU *

   MOV   DPTR,#MCPulses
   MOV   A,#$FF
   MOVX  [DPTR],A
   INC   DPTR
   MOVX  [DPTR],A
   MOV   R4,#UNP_DONE
   ORL   ErrFlags,#ERR_SBAD
   SJMP  ?Quit

*--- see if we are done with the record
*    if so, verify that the pulse counter is also zero
*    then decrement record counter to account for this one
*    also save pointer to next record in case this is the last one

?EndLp   EQU *

   LCJB  R4,#UNP_DONE,?UnPack          ; if below, jump back to top

   JNZRR R6,R7,?badx                   ; nonzero count is bad exit

   DecRR R3,R2                         ; record count - 1

   PUSH  DPL                           ; save pointer to next record
   PUSH  DPH
   MOV   B,DPH
   MOV   A,DPL
   MOV   DPTR,#pMCNext
   MOVX  [DPTR],A
   INC   DPTR
   MOV   A,B
   MOVX  [DPTR],A
   POP   DPH
   POP   DPL

*--- record is unpacked correctly, check to see if it is a match
*    simply fall out the bottom if so, with all variables already set up

   MOV   R0,#-5                        ; set up pointer to SeekID on stack
   LCALL ?auto0                        ; [R0] -> low byte, [R0+1] -> high

   MOV   A,[R0]                        ; fetch low byte of desired ID
   INC   R0
   LCJNE A,MCRecID,?GetRec
   MOV   A,[R0]                        ; if match, check high byte
   LCJNE A,MCRecID+1,?GetRec

?Quit    EQU   *

}

Done:

   if (REPORT_DECOMP & ReportMode) {
      Nputstr("--- Dump after decompression\n");
      MCDumpState();
   }

   return MCPulses;

}


/*---------------------------------------------------------------------------*/
/* Dump current state                                                        */

#define DUMPSIZE 1024

MCDumpState() {
int Index;
int SubIndex;
BYTE *pEntry;

   Nputstr("MCIR status:\n");

   sprintf(CmdRespBuff,
     " pMCData=%04x pMCThis=%04x pMCNext=%04x \n",
     pMCData,pMCThis,pMCNext);
   Nputstr(CmdRespBuff);

   sprintf(CmdRespBuff,
     " MCRecs=%04x MCRecID=%04x MCPulses=%04x\n",
     MCRecs,MCRecID,MCPulses);
   Nputstr(CmdRespBuff);

   Nputstr("Timer buffer\n");
   for (Index=0; Index < DUMPSIZE; Index += 24) {
      pEntry = (BYTE *)&MCTimes[Index];
      sprintf(CmdRespBuff," %02x%04x %02x%04x ",
              *(pEntry+2),*(WORD *)(pEntry+0),
              *(pEntry+5),*(WORD *)(pEntry+3));
      Nputstr(CmdRespBuff);
      sprintf(CmdRespBuff," %02x%04x %02x%04x ",
              *(pEntry+8),*(WORD *)(pEntry+6),
              *(pEntry+11),*(WORD *)(pEntry+9));
      Nputstr(CmdRespBuff);
      sprintf(CmdRespBuff," %02x%04x %02x%04x ",
              *(pEntry+14),*(WORD *)(pEntry+12),
              *(pEntry+17),*(WORD *)(pEntry+15));
      Nputstr(CmdRespBuff);
      sprintf(CmdRespBuff," %02x%04x %02x%04x\n",
              *(pEntry+20),*(WORD *)(pEntry+18),
              *(pEntry+23),*(WORD *)(pEntry+21));
      Nputstr(CmdRespBuff);
   }

   Nputstr("RAM record storage:\n");
   pEntry = pMCRAM;
   for (Index = 0; Index < DUMPSIZE; Index += 16) {
      sprintf(CmdRespBuff," %04x ",pEntry);
      Nputstr(CmdRespBuff);
      for (SubIndex = 0; SubIndex < 16; ++SubIndex) {
         sprintf(CmdRespBuff," %02x",*pEntry++);
         Nputstr(CmdRespBuff);
      }
      Nputch('\n');
   }


}

/*---------------------------------------------------------------------------*/
/* Send single line of Intel HEX data                                        */
/* Inputs are:                                                               */
/*  pointer to the data                                                      */
/*  number of bytes to send                                                  */
/*  pointer to starting address for this chunk                               */
/* Uses record type 00, so you get to handle all the other types yourself    */
/* You get to limit the line length to whatever you need...                  */

MCSendHex(pData,NumBytes,BaseAddr)
BYTE *pData;
WORD NumBytes;
WORD BaseAddr;
{
BYTE CheckSum;
BYTE XChar;

   if (chkch()) {
      if (XOFF == getch()) {
         do {
            XChar = getch();
            CPLBIT(HEARTBEAT);                     /* indicate we are stuck  */
         } while (XON != XChar);
      }
   }

   sprintf(CmdRespBuff,":%02x%04x00",NumBytes,BaseAddr);
   Nputstr(CmdRespBuff);

   CheckSum = NumBytes + (BaseAddr >> 8) + (BaseAddr & 0x00ff);

   while (NumBytes--) {
      sprintf(CmdRespBuff,"%02x",*pData);
      Nputstr(CmdRespBuff);
      CheckSum += *pData++;
   }

   sprintf(CmdRespBuff,"%02x\n",-CheckSum);
   Nputstr(CmdRespBuff);

   return;

}


/*---------------------------------------------------------------------------*/
/* Dump the signals stored in a data structure in HEX format                 */
/* We try to unpack all the records just to see if the thing is valid        */
/* The BaseAddr param sets the HEX line addresses, nothing more...           */

#define TRC_DUMPxxx

MCDumpData(pIRData,BaseAddr)
BYTE *pIRData;
WORD BaseAddr;
{
BYTE LineLen;
BYTE *pLast;
int  NumPulses;

   pMCData = pIRData;

#ifdef TRC_DUMP
   printf("Testing IR data structure at %04x/%04x... ",pIRData,pMCData);
#endif

   NumPulses = MCFetchRec(MAXRECNUM);              /* -1 means problem       */


#ifdef TRC_DUMP
   printf(" pulse count %d, records %d\n",NumPulses,MCRecs);
   Nputstr("--- Dump before structure test\n");
   MCDumpState();
#endif

   if (MCRecs && (NumPulses >= 0)) {
      pLast = pMCNext + 2;                         /* include two end flags  */
      while (pIRData < pLast) {
         CPLBIT(HEARTBEAT);
         LineLen = min(HEXLINELEN,pLast-pIRData);
         MCSendHex(pIRData,LineLen,BaseAddr);
         BaseAddr += LineLen;
         pIRData += LineLen;
      }
   }
   else {
      CmdShowMsg("*** No IR data\n");
   }

   Nputstr(":00000001FF\n");           /* standard end record for all cases  */

   return;

}


/*---------------------------------------------------------------------------*/
/* Get a single line of (presumably) HEX data from the serial port w/echo    */
/* This needs to twiddle the heartbeat output while waiting...               */
/* Returns the number of characters in the line                              */
/*  -- returns -1 if entry was cancelled with Ctrl-C or Esc                  */

#define TRC_GETLINExxx

int MCGetLine(pBuffer,MaxLen)
char *pBuffer;
int  MaxLen;
{

int NumChars;

   NumChars = 0;

   while (NumChars < MaxLen) {
      CPLBIT(HEARTBEAT);
      if (chkch()) {
         *pBuffer = getche();                      /* echoes incoming chars  */
         switch (*pBuffer) {
         case CANCEL :
         case ESC :
            NumChars = -1;
#ifdef TRC_GETLINE
            printf("Bailing out of MCGetLine, char is %02x\n",
                    *pBuffer);
#endif
            /* fall through */
         case '\n' :
#ifdef TRC_GETLINE
            printf("End of line with length %d\n",
                    NumChars);
#endif
            goto Done;
         }
         ++pBuffer;
         ++NumChars;
      }
   }

Done:
#ifdef TRC_GETLINE
            printf("Char count %d, last char %02x\n",
                    NumChars,*pBuffer);
#endif

   *pBuffer = 0;                                   /* stomp last char        */

   return NumChars;

}




/*---------------------------------------------------------------------------*/
/* Process a single line of Intel HEX data to the IR RAM data area           */
/* Returns data length:  -1 for error, 0 for end, >0 for normal records      */
/* Presumes caller is handling all the XON/XOFF hocus pocus                  */
/* This replaces the incoming ASCII data with translated binary              */
/*  offset 0 = 1 byte of data length                                         */
/*         1 = 2 bytes of address in int format (low byte first)             */
/*         3 = start of data, one byte per value                             */
/* MUST be given a valid hex line to avoid errors; filter trash out early!   */

#define BHEX_LEN  0
#define BHEX_ADDR 1
#define BHEX_DATA 3

#define TRC_PHEXxxx

int MCProcessHex(pText)
char *pText;
{
int  DataLen;
WORD Address;
BYTE RecType;
BYTE CheckSum;
BYTE *pByte;
char *pChar;
BYTE NumBytes;

   pByte = pText;                                  /* set up output pointer  */

   if (':' != *pText) {
      sprintf(CmdRespBuff,"*** Intel HEX line must start with colon\n[%s]\n",
         pText);
      Nputstr(CmdRespBuff);
      DataLen = -1;
      goto Done;
   }

   pChar = pText + 1;                              /* need XRAM copy         */

#ifdef TRC_PHEX
   sprintf(CmdRespBuff,"HEX line [%s]\n",pChar);
   Nputstr(CmdRespBuff);
#endif

   DataLen = GetHexits(&pChar,2);                  /* parse the header       */
   Address = GetHexits(&pChar,4);
   RecType = GetHexits(&pChar,2);

#ifdef TRC_PHEX
   sprintf(CmdRespBuff,"DataLen %02x  Address %04x  RecType %02x\n",
      DataLen,Address,RecType);
   Nputstr(CmdRespBuff);
#endif

   *pByte++ = (BYTE)DataLen;                       /* store header info      */
   *pByte++ = (BYTE)Address;
   *pByte++ = (BYTE)(Address >> 8);

   CheckSum = (BYTE)DataLen +
              (Address >> 8) + (Address & 0x00ff) +
              RecType;

#ifdef TRC_PHEX
   printf("Checksum is %02x\n",CheckSum);
#endif

   for (NumBytes=DataLen; NumBytes>0; --NumBytes) {         /* do the data   */
      *pByte = GetHexits(&pChar,2);
      CheckSum += *pByte;
#ifdef TRC_PHEX
      printf("Data %02x CheckSum %02x\n",*pByte,CheckSum);
#endif
      ++pByte;
   }

   CheckSum += GetHexits(&pChar,2);                /* finish off the line    */

   if (CheckSum) {                                 /* bail out if problem    */
      sprintf(CmdRespBuff,"*** Checksum error: %02x\n",CheckSum);
      CmdShowMsg(CmdRespBuff);
      DataLen = -1;
   }

   if (RecType) {                                  /* bail out if problem    */
      DataLen = -1;
   }

Done:
   return DataLen;

}


/*---------------------------------------------------------------------------*/
/* Load RAM from Intel HEX file                                              */
/* Discards non-HEX lines and issues message...                              */

#define TRC_LDRAMxxx

MCLoadRAM() {
int DataLen;
int  AddrOffset;
WORD LineBase;

/*--- get first line to find address offset value                            */
/*    we wait for a leading colon, but bail out on cancel or error lines     */

   do {
      Nputch(XON);                                 /* allow input            */
      Nputstr(HexPrompt);                          /* issue prompt           */
      Nputch(NET_FORCEOUT);                        /* force output           */
      DataLen = MCGetLine(HexLine,sizeof(HexLine));
      if (DataLen < 0) {
         goto Done;
      }
      if (DataLen && (':' != *HexLine)) {          /* junk non-HEX lines     */
         DataLen = 0;
         continue;
      }
      Nputch(XOFF);                                /* shut off input         */
      Nputch(NET_FORCEOUT);                        /* force output           */
   } while (!DataLen);                             /* wait for alleged HEX   */

   DataLen = MCProcessHex(HexLine);                /* check & convert        */
   if (DataLen < 0) {                              /* bail on errors         */
      goto Done;
   }

   AddrOffset = pMCRAM - *(WORD *)(HexLine + BHEX_ADDR);    /* do first line */
#ifdef TRC_LDRAM
   printf("Address offset %04x\n",AddrOffset);
#endif

   move(pMCRAM,HexLine+BHEX_DATA,*(HexLine+BHEX_LEN));

   do {
      Nputch(XON);                                 /* allow input            */
  /*  Nputch('\n');   */
      Nputstr(HexPrompt);                          /* issue prompt           */
      Nputch(NET_FORCEOUT);                        /* force output           */
      DataLen = MCGetLine(HexLine,sizeof(HexLine));
      if (DataLen < 0) {                           /* bail on error or break */
         goto Done;
      }
      if (DataLen && (':' != *HexLine)) {          /* find non-HEX lines     */
         DataLen = 0;
         continue;
      }
      if (!DataLen) {
         continue;
      }
      Nputch(XOFF);                                /* shut off input         */
      Nputch(NET_FORCEOUT);                        /* force output           */


      DataLen = MCProcessHex(HexLine);             /* check & convert        */
      if (DataLen <= 0) {                          /* bail on errors or end  */
         continue;
      }

      LineBase = AddrOffset + *(WORD *)(HexLine + BHEX_ADDR);

      if (LineBase < pMCRAM) {                     /* protect our data!      */
         sprintf(CmdRespBuff,"*** Line address below start: %04x\n",
                 *(WORD *)(HexLine + BHEX_ADDR));
         CmdShowMsg(CmdRespBuff);
         DataLen = 0;                              /* force iteration        */
         continue;
      }

      move(LineBase,HexLine+BHEX_DATA,*(HexLine+BHEX_LEN));
#ifdef TRC_LDRAM
      printf("Move to %04x from %04x length %04x\n",
             LineBase,HexLine+BHEX_DATA,*(HexLine+BHEX_LEN));
#endif

   } while (DataLen >= 0);

Done:

   Nputch(XON);                  /* re-enable input after termination        */
   Nputch('\n');                 /* force new line after this stuff          */

   return;

}



/*---------------------------------------------------------------------------*/
/* Convert timing data from decompressed values to timer reloads and wraps   */
/* This presumes the buffer is loaded with valid data...                     */
/* Shift data to regain x1 scaling                                           */
/* Convert two LSBs to 2s complement for initial timer loading               */

WORD MCCvtToTimer() {

   if (REPORT_TRANSMIT & ReportMode) {
      Nputstr("Converting to timer reload values... ");
   }

asm {

   MOV   DPTR,#MCPulses                ; fetch number of pulses to convert
   MOVX  A,[DPTR]                      ; low byte
   CLR   C
   RLC   A                             ; convert to time slots
   MOV   R0,A
   INC   DPTR
   MOVX  A,[DPTR]                      ; high byte
   RLC   A
   MOV   R1,A

   MOV   DPTR,#MCTimes                 ; aim at output buffer

*--- for each time value...

?CvtTo0
   MOV   R2,DPL                        ; save starting point
   MOV   R3,DPH

   MOVX  A,[DPTR]                      ; fetch bytes, low first
   INC   DPTR
   MOV   R4,A
   MOVX  A,[DPTR]
   INC   DPTR
   MOV   R5,A
   MOVX  A,[DPTR]
   INC   DPTR
   MOV   R6,A

*--- shift left to rescale

   MOV   B,#3
   CLR   C

?CvtTo1
   MOV   A,R4
   RLC   A
   MOV   R4,A
   MOV   A,R5
   RLC   A
   MOV   R5,A
   MOV   A,R6
   RLC   A
   MOV   R6,A
   DJNZ  B,?CvtTo1

*--- convert low word to 2s complement for timer reload
*     adjust wrap count to compensate for trailing DJNZ

   MOV   A,R4                          ; timer reloads
   CPL   A
   ADD   A,#1
   MOV   R4,A
   MOV   A,R5
   CPL   A
   ADDC  A,#0
   MOV   R5,A

   INC   R6                            ; wrap counter

*--- store new value back into buffer in same location

   MOV   DPL,R2                        ; restore starting point
   MOV   DPH,R3

   MOV   A,R4
   MOVX  [DPTR],A                      ; store bytes, low first
   INC   DPTR
   MOV   A,R5
   MOVX  [DPTR],A
   INC   DPTR
   MOV   A,R6
   MOVX  [DPTR],A
   INC   DPTR

   DecRR R1,R0                         ; tick counter and continue
   JNZRR R1,R0,?CvtTo0

}

   return;

}


/*---------------------------------------------------------------------------*/
/* Convert timing data from raw timer values to per-pulse values             */
/* Includes a fudge factor for the timer capture duration                    */
/*  as well as one to correct the ON/OFF duty cycle                          */
/* Timer captures are ascending counts starting from zero with wrap tally    */
/* Output is plopped in two low-order bytes of array, MSB cleared            */
/* These values are not compressed and formatted to be sent again...         */

#define FIX_LOOP 0

WORD MCCvtFromTimer() {

   if (REPORT_COMPRESS & ReportMode) {
      Nputstr("  Converting from timer capture values\n");
   }

asm {

   MOV   DPTR,#MCPulses                ; fetch number of pulses to convert
   MOVX  A,[DPTR]                      ; low byte
   CLR   C
   RLC   A                             ; convert to time slots
   MOV   R0,A
   INC   DPTR
   MOVX  A,[DPTR]                      ; high byte
   RLC   A
   MOV   R1,A

   MOV   DPTR,#MCTimes                 ; aim at buffer

   MOV   R2,#-18                       ; preset previous time value
   MOV   R3,#$FF                       ;  ... with offset for first pulse!
   MOV   R4,#$FF

   SETB  F0FLAG                        ; start with ON time subtraction

*--- for each time value...

?CvtFr0
   PUSH  DPL                           ; save starting point
   PUSH  DPH

   MOVX  A,[DPTR]                      ; fetch bytes, low first
   INC   DPTR
   MOV   R5,A
   MOVX  A,[DPTR]
   INC   DPTR
   MOV   R6,A
   MOVX  A,[DPTR]                      ; this is the wrap tally
   MOV   R7,A

*--- subtract previous time to get incremental time

   MOV   A,R5
   CLR   C
   SUBB  A,R2
   MOV   R5,A
   MOV   A,R6
   SUBB  A,R3
   MOV   R6,A
   MOV   A,R7
   SUBB  A,R4
   MOV   R7,A


*--- add loop overhead correction

   MOV   A,R5
   ADD   A,#FIX_LOOP
   MOV   R5,A
   MOV   A,R6
   ADDC  A,#0
   MOV   R6,A
   MOV   A,R7
   ADDC  A,#0
   MOV   R7,A

*--- add or subtract duty cycle correction factor
*    if the subtraction underflows, then that half of the pulse was short
*      (so we discard the adjustment for that half and garble the timing)

   JB    F0FLAG,?AdjON

   MOV   A,R5
   ADD   A,MCDuty
   MOV   R5,A
   MOV   A,R6
   ADDC  A,#0
   MOV   R6,A
   MOV   A,R7
   ADDC  A,#0
   MOV   R7,A

   SETB  F0FLAG

   SJMP  ?AdjEnd

?AdjON

   MOV   A,R5
   CLR   C
   SUBB  A,MCDuty
   MOV   R5,A
   MOV   A,R6
   SUBB  A,#0
   MOV   R6,A
   MOV   A,R7
   SUBB  A,#0
   MOV   R7,A

   JNC   ?AdjON1                       ; no carry means pulse >= adjustment

   MOV   A,R5                          ; if less, then add adjustment back in
   ADD   A,MCDuty
   MOV   R5,A
   MOV   A,R6
   ADDC  A,#0
   MOV   R6,A
   MOV   A,R7
   ADDC  A,#0
   MOV   R7,A

?AdjON1

   CLR   F0FLAG

?AdjEnd

*--- divide result by 8 and preserve remainder for next step

   MOV   B,#3
   CLR   A
   PUSH  A
   CLR   C

?CvtFr1
   MOV   A,R7
   RRC   A
   MOV   R7,A
   MOV   A,R6
   RRC   A
   MOV   R6,A
   MOV   A,R5
   RRC   A
   MOV   R5,A
   POP   A
   RRC   A
   PUSH  A
   DJNZ  B,?CvtFr1

   POP   A
   SWAP  A                       ; move remainder to low nibble
   CLR   C                       ; align remainder to bit 0
   RRC   A
   MOV   B,A                     ; save for later

*--- store incremental time back into buffer at same location
*    extract the cumulative time value for next iteration

   POP   DPH                           ; restore starting point
   POP   DPL

   MOVX  A,[DPTR]                      ; get cumulative time again
   MOV   R2,A
   MOV   A,R5
   MOVX  [DPTR],A                      ; store delta time in its place
   INC   DPTR
   MOVX  A,[DPTR]
   MOV   R3,A
   MOV   A,R6
   MOVX  [DPTR],A
   INC   DPTR
   MOVX  A,[DPTR]
   MOV   R4,A
   MOV   A,R7
   MOVX  [DPTR],A
   INC   DPTR

*--- deduct remainder from previous time, so as to add it to next time

   MOV   A,R2
   CLR   C
   SUBB  A,B
   MOV   R2,A
   MOV   A,R3
   SUBB  A,#0
   MOV   R3,A
   MOV   A,R4
   SUBB  A,#0
   MOV   R4,A

*--- tick the loop counter and continue

   DecRR R1,R0
   LJNZRR R1,R0,?CvtFr0


}

   return;

}




/*---------------------------------------------------------------------------*/
/* Compress timing values into standard format into target buffer            */
/* Presumes the timing buffer has two-byte per-pulse timing values           */
/*  stored in low-order two bytes of each three-byte array entry             */
/* pTarget points to the start of the new record area                        */
/* MaxLen indicates the number of bytes available                            */
/* pMCData must point to start of the whole data area                        */
/* MCRecID must contain record number                                        */
/* MCPulses must hold number of pulses in MCTimes array                      */
/* Returns number of pulses compressed or -1 for problem                     */

int MCCompress(pTarget,MaxLen)
BYTE *pTarget;
int  MaxLen;
{

   if ((MCPulses <= 0) || (MaxLen < 12))  {
      Nputstr("*** No pulses to compress or RAM is obviously too full\n");
      MCPulses = -1;
      goto Done;
   }

   if (REPORT_COMPRESS & ReportMode) {
      Nputstr("Starting compression\n");
      MCDumpState();
   }

asm {

   MOV   DPTR,#MCPulses                ; fetch pulse counter
   MOVX  A,[DPTR]
   INC   DPTR
   MOV   R4,A                          ;  ... into R5:R4
   MOVX  A,[DPTR]
   MOV   R5,A

   MOV   R0,#-7                        ; fetch pTarget
   LCALL ?auto0
   MOV   DPL,[R0]                      ;  ... into DPTR
   INC   R0
   MOV   DPH,[R0]
   INC   R0                            ; point to MaxLen on stack
   MOV   ?R2,[R0]                      ;  ... set up R3:R2
   INC   R0
   MOV   ?R3,[R0]

   MOV   MCAddr,#HIGH(MCTimes)         ; aim at source buffer
   MOV   R1,#LOW(MCTimes)              ;  ... with MCAddr:R1

*--- load the record header

   MOV   A,MCRecID                     ; record ID
   MOVX  [DPTR],A
   INC   DPTR
   MOV   A,MCRecID+1
   MOVX  [DPTR],A
   INC   DPTR

   MOV   A,R4                          ; pulse count
   MOVX  [DPTR],A
   INC   DPTR
   MOV   A,R5
   MOVX  [DPTR],A
   INC   DPTR

*--- copy first pulse intact

   MOV   R0,#2                         ; move two source entries

?MvW0    EQU *

   MOV   B,#2                          ; move two bytes per entry
?MvW1    EQU *
   MOV   P2,MCAddr
   MOVX  A,[R1]                        ; move data byte
   InCRR MCAddr,R1
   MOVX  [DPTR],A
   INC   DPTR
   DJNZ  B,?MvW1

   IncRR MCAddr,R1                         ; step over third source byte

   DJNZ  R0,?MvW0

   DecRR R5,R4                         ; account for first pulse

   SubRR R3,R2,#0,#8                   ; account for header and pulse

*--- set up for byte compression

P_BYTES  SET   0
P_WORDS  SET   1

   MOV   R0,#P_BYTES

   CLR   A                             ; initialize byte run counter
   MOVX  [DPTR],A

   MOV   R6,DPL                        ; save pointer to it for later
   MOV   R7,DPH

   INC   DPTR                          ; step to first data byte in block


*---------------------------------------------
*--- main compression loop, run once per pulse

*--- check for space and pulses remaining

?CTopLp  EQU   *

   MOV   A,R5                          ; pulses < 0 is failure
   LJB   A.7,?MCCErr
   ORL   A,R4                          ; pulses = 0 is the end!
   LJZ   ?MCCDone

   MOV   A,R3                          ; space < 0 is failure
   LJB   A.7,?MCCErr
   ORL   A,R2                          ; space = 0 is failure
   LJZ   ?MCCErr

*--- decide how to compress the data at hand

*--- we are compressing a run of bytes
*    if both source values are < 256, continue with bytes
*    otherwise switch to a word run

   LCJNE R0,#P_BYTES,?CWds

   PUSH  MCAddr                        ; save pointer to source values
   PUSH  ?R1                           ;  in case this run fails

   MOV   P2,MCAddr
   MOVX  A,[R1]                        ; fetch low byte of ON time
   IncRR MCAddr,R1
   MOV   R0,A                          ; save in case we need it
   MOV   P2,MCAddr
   MOVX  A,[R1]                        ; fetch high byte of ON time
   AddRR MCAddr,R1,#0,#2               ; step to next time

   JNZ   ?Byt2Wd                       ; high nonzero means not a byte!

   MOV   P2,MCAddr
   MOVX  A,[R1]                        ; fetch low byte of OFF time
   IncRR MCAddr,R1
   MOV   B,A
   MOV   P2,MCAddr
   MOVX  A,[R1]                        ; fetch high byte of OFF time
   AddRR MCAddr,R1,#0,#2               ; step to next time

   JNZ   ?Byt2Wd                       ; high nonzero means not a byte!

   MOV   A,R0                          ; store ON time byte
   MOVX  [DPTR],A
   INC   DPTR
   MOV   A,B                           ; store OFF time byte
   MOVX  [DPTR],A
   INC   DPTR

   PUSH  DPL                           ; increment run length counter
   PUSH  DPH

   MOV   DPL,R6
   MOV   DPH,R7
   MOVX  A,[DPTR]
   INC   A
   MOVX  [DPTR],A
   CJE   A,#$FF,?Byt2Wd1               ; if now FF, must terminate run

   POP   DPH
   POP   DPL

   POP   A                             ; discard source pointer
   POP   A

   AddRR R3,R2,#$FF,#$FE               ; subtract 2 bytes from remaining RAM
   DecRR R5,R4                         ; and account for one more pulse

   MOV   R0,#P_BYTES                   ; continue byte run
   LJMP  ?CBotLp

*--- must switch to word run

?Byt2Wd1
   POP   DPH                           ; restore output pointer
   POP   DPL

?Byt2Wd

   MOV   R0,#P_WORDS                   ; flag it

   POP   ?R1                           ; restore source pointer
   POP   MCAddr

   CLR   A                             ; initialize new run length counter
   MOVX  [DPTR],A

   MOV   R6,DPL                        ; save pointer to it for later
   MOV   R7,DPH

   DecRR R3,R2                         ; one less byte for data...

   INC   DPTR                          ; step to first data byte in block

   LJMP  ?CBotLp

*--- we are compressing a run of words
*    if either source values is >= 256, continue with words
*    otherwise switch to a byte run

?CWds
   LCJNE R0,#P_WORDS,?CBad

   PUSH  MCAddr                        ; save pointer to source values
   PUSH  ?R1                           ;  in case this run fails

   PUSH  ?R6                           ; save run length to get temp space
   PUSH  ?R7

   MOV   P2,MCAddr
   MOVX  A,[R1]                        ; fetch low byte of ON time
   IncRR MCAddr,R1
   MOV   R6,A                          ; save in case we need it
   MOV   P2,MCAddr
   MOVX  A,[R1]                        ; fetch high byte of ON time
   AddRR MCAddr,R1,#0,#2               ; step to next time
   MOV   R7,A

   MOV   P2,MCAddr
   MOVX  A,[R1]                        ; fetch low byte of OFF time
   IncRR MCAddr,R1
   MOV   B,A
   MOV   P2,MCAddr
   MOVX  A,[R1]                        ; fetch high byte of OFF time
   AddRR MCAddr,R1,#0,#2               ; step to next time
   MOV   R0,A

   ORL   A,R7                          ; combine both high bytes
   JZ    ?Wd2Byt1                      ; both zero means switch to byte run

   MOV   A,R6                          ; store low ON time byte
   MOVX  [DPTR],A
   INC   DPTR
   MOV   A,R7                          ; store high ON time byte
   MOVX  [DPTR],A
   INC   DPTR

   MOV   A,B                           ; store low OFF time byte
   MOVX  [DPTR],A
   INC   DPTR
   MOV   A,R0                          ; store high OFF time byte
   MOVX  [DPTR],A
   INC   DPTR

   POP   ?R7                           ; recover run length pointer
   POP   ?R6

   PUSH  DPL                           ; increment run length counter
   PUSH  DPH

   MOV   DPL,R6
   MOV   DPH,R7
   MOVX  A,[DPTR]
   INC   A
   MOVX  [DPTR],A
   CJE   A,#$FF,?Wd2Byt                ; if now FF, must terminate run

   POP   DPH                           ; recover output pointer
   POP   DPL

   POP   A                             ; discard source pointer
   POP   A

   AddRR R3,R2,#$FF,#$FC               ; subtract 4 bytes from remaining RAM
   DecRR R5,R4                         ; and account for one more pulse

   MOV   R0,#P_WORDS                   ; continue word run
   LJMP  ?CBotLp

*--- must switch to byte run

?Wd2Byt1

   POP   ?R7                           ; recover run length pointer
   POP   ?R6

?Wd2Byt

   MOV   R0,#P_BYTES                   ; flag it

   POP   ?R1                           ; restore source pointer
   POP   MCAddr

   CLR   A                             ; initialize new run length counter
   MOVX  [DPTR],A

   MOV   R6,DPL                        ; save pointer to it for later
   MOV   R7,DPH

   DecRR R3,R2                         ; one less byte for data...

   INC   DPTR                          ; step to first data byte in block

   SJMP  ?CBotLp

*--- we have a bogus compression selector!

?CBad
   MOV   R3,#0
   MOV   R2,#0
   SJMP  ?MCCdone

*--- bottom of the compression loop
*    all the acceptance tests are done at the top...

?CBotLp  EQU *
   LJMP  ?CTopLp

*--- ran into an error

?MCCErr  EQU *
   MOV   DPTR,#MCPulses
   MOV   A,#$FF
   MOVX  [DPTR],A
   INC   DPTR
   MOVX  [DPTR],A
   SJMP  ?MCCexit

*--- done with compression, add terminators and save pointer to end

?MCCdone EQU *

   CLR   A
   MOV   B,#4

?MCCfill
   MOVX  [DPTR],A
   INC   DPTR
   DJNZ  B,?MCCfill

   MOV   A,DPL
   MOV   B,DPH
   MOV   DPTR,#pMCNext
   MOVX  [DPTR],A
   INC   DPTR
   MOV   A,B
   MOVX  [DPTR],A

?MCCExit EQU *

}

   if (MCPulses > 0) {
      if (REPORT_COMPRESS & ReportMode) {
         Nputstr("Increment record counter\n");
      }
      MCRecs = ++(*(WORD *)pMCRAM);    /* account for new record             */

   }
   else {
      if (REPORT_COMPRESS & ReportMode) {
         Nputstr("Removing bad record\n");
      }
      memset(pTarget,0,MaxLen);
   }

Done:

   if (REPORT_COMPRESS & ReportMode) {
      Nputstr("  compression finished\n");
      MCDumpState();
   }

   return MCPulses;

}


/*---------------------------------------------------------------------------*/
/* Send MCIR signal directly from buffer                                     */

MCSendBuffer() {

   if (REPORT_TRANSMIT & ReportMode) {
   Nputstr("--- Dump before transmission\n");
   MCDumpState();
   }

asm {

   MOV   DPTR,#MCPulses                ; get pulse counter
   MOVX  A,[DPTR]
   INC   DPTR
   CLR   C                             ; times 2 to get entry counter
   RLC   A
   MOV   R2,A
   MOVX  A,[DPTR]
   RLC   A
   MOV   R3,A

*--- set up the first pulse for loop entry

   MOV   DPTR,#MCTimes                 ; point to array base

   MOVX  A,[DPTR]                      ; low timer reload
   INC   DPTR
   MOV   R4,A
   MOVX  A,[DPTR]                      ; high timer reload
   INC   DPTR
   MOV   R5,A
   MOVX  A,[DPTR]                      ; wrap counter
   INC   DPTR
   MOV   R6,A

*--- make sure the other IR I/O is not active...
*    this will clear when reception or transmission is done

   MOV   A,#IR_IDLE
?IRlp
   CPL   HEARTBEAT
   CJNE  A,IRState,?IRlp

   PUSH  IE                            ; save existing interrupt state
   CLR   EA                            ; shut them off entirely

*--- repeat for each pulse in the array...

   CLR   ITRANS                        ; this should already be OFF

?MCloop

   CLR   TR0                           ; 1 stop the timer
   MOV   TL0,R4                        ; 2 load partial count
   MOV   TH0,R5                        ; 2
   CLR   TF0                           ; 1 clear wrap flag
   SETB  TR0                           ; 1 fire it up

   CPL   ITRANS                        ; 1 flip the LED state

   MOV   B,R6                          ; set up wrap counter

   MOVX  A,[DPTR]                      ; set up next timer values
   INC   DPTR
   MOV   R4,A
   MOVX  A,[DPTR]                      ; high timer reload
   INC   DPTR
   MOV   R5,A
   MOVX  A,[DPTR]                      ; wrap counter
   INC   DPTR
   MOV   R6,A

   DecRR R3,R2                         ; tick array entry counter
   MOV   A,R2                          ; set up for after-loop test...
   ORL   A,R3

?MCinner
   CPL   HEARTBEAT                     ; 1
   JNB   TF0,?MCinner                  ; 2 wait for timer wraps
   CLR   TF0                           ; 1  ... continue for full 64K count
   DJNZ  B,?MCinner                    ; 2  ... until all wraps are done

   JNZ   ?MCloop                       ; 2 use values set up before ?MCinner

*--- done, go home

   CLR   ITRANS                        ; this should already be OFF

   CLR   TR0                           ; restart normal timing
   MOV   TL0,T0Reload
   MOV   TH0,T0Reload+1
   CLR   TF0
   SETB  TR0

   POP   IE                            ; restore interrupts

}

   return;

}



/*---------------------------------------------------------------------------*/
/* Unpack and send stored MCIR signal                                        */

MCSendSignal(RecID)
int RecID;
{

   if ((RecID < 1) ||
       (RecID == (MAXRECNUM+1)) ||
       (RecID >= 2*(MAXRECNUM+1))) {
      CmdShowMsg("*** Bad MCIR record number\n");
      goto Done;
   }

   if (RecID > MAXRECNUM) {                        /* in EPROM?              */
      pMCData = pMCEPROM;
      RecID -= MAXRECNUM + 1;
   }
   else {
      pMCData = pMCRAM;                            /* or RAM...              */
   }

   switch (MCFetchRec(RecID)) {
   case -1 :
      CmdShowMsg("*** IR data not valid\n");
      goto Done;
   case 0 :
      CmdShowMsg("*** IR record not found\n");
      goto Done;
   }

   if (MCPulses <=0) {
      sprintf(CmdRespBuff,"*** No pulses: %d\n",MCPulses);
      Nputstr(CmdRespBuff);
      goto Done;
   }

   MCCvtToTimer();                     /* convert to timer reload values     */

   MCSendBuffer();                     /* and send it out...                 */

Done:
   return;

}
/*---------------------------------------------------------------------------*/
/* Capture raw IR signal into timing buffer                                  */
/* MCPulses holds the number of ON/OFF pairs captured                        */
/* This truncates downward if the IR signal is stuck ON                      */

MCCapRaw()
{

   memset(MCTimes,0,sizeof(MCTimes));  /* for easy visibility                */

   SerWaitOutDone();                   /* prevent stall during disable       */
   SerFlushIn();                       /* flush any pending chars            */

asm {

*--- set up counters and pointers for the capture loop

   MOV   A,#LOW(MAXPULSES)             ; get number of time samples to take
   CLR   C
   RLC   A
   MOV   R2,A
   MOV   A,#HIGH(MAXPULSES)
   RLC   A
   MOV   R3,A

   MOV   DPTR,#MCTimes                 ; aim at the recording buffer

*--- make sure the other IR I/O is not active...
*    this will clear when reception or transmission is done

   MOV   A,#IR_IDLE
?Raw0
   CPL   HEARTBEAT
   CJNE  A,IRState,?Raw0

*--- wait for an IR signal or cancel keystroke

   SETB  IRECEIVE                      ; ensure it is an input...
   CLR   F0FLAG                        ; indicate no timeout

?Raw1
   MOV   A,RR_cLev                     ; 1 monitor serial character count
   LJNZ  ?RawDone                      ; 2 and bail out if any show up
   CPL   HEARTBEAT                     ; 1 indicate we are alive
   JB    IRECEIVE,?Raw1                ; 2 high with no IR signal
   CLR   FlagMCSaveIR                  ; 1 force memory low to match

*--- hold off interrupts and start timer at zero

   PUSH  IE                            ; 2 save existing interrupt state
   CLR   EA                            ; 1 shut them off entirely

   CLR   TR0                           ; 1 reset the timer
   MOV   TL0,#0                        ; 2
   MOV   TH0,#0                        ; 2
   CLR   TF0                           ; 1
   SETB  TR0                           ; 1

*--- repeat for each time entry in the array or until the signal times out

   MOV   B,#0                          ; 2 clear wrap counter accumulator

?RawLp

   JB    FlagMCSaveIR,?RawU0           ; 2 which bit did we have last time?

?RawD0
   CPL   HEARTBEAT                     ; 1 indicate we are alive
   JNB   TF0,?RawD1                    ; 2 check for timeout
   CLR   TF0                             1
   INC   B                               1
   JB    B.3,?RawTO                    ; 2 time out after 8 wraps
?RawD1
   JNB   IRECEIVE,?RawD0               ; 2 spin if still down
   SETB  FlagMCSaveIR                  ; 1 remember new bit state
   SJMP  ?RawEdge                      ; 2

?RawTO
   ORL   ErrFlags,#$20                 ; record for debugging
   SETB  F0FLAG                        ; and force loop exit
   SJMP  ?RawEdge                      ; after normal recording

?RawU0
   CPL   HEARTBEAT                     ; 1 indicate we are alive
   JNB   TF0,?RawU1                    ; 2 check for timeout
   CLR   TF0                           ; 1
   INC   B                             ; 1
   JB    B.3,?RawTO                    ; 2 time out after 8 wraps
?RawU1
   JB    IRECEIVE,?RawU0               ; 2 spin if still up
   CLR   FlagMCSaveIR                  ; 1 remember new bit state

*--- edge detected, store current timestamp

?RawEdge

   CLR   TR0                           ; 1 freeze timer
   MOV   A,B                           ; 1 pick up wrap counter
   MOV   C,TF0                         ; 1 incorporate possible wrap
   ADDC  A,#0                          ; 1
   MOV   B,A                           ; 1 save it for later
   MOV   R4,TL0                        ; 2
   MOV   R5,TH0                        ; 2
   CLR   TF0                           ; 1 clear timer wrap flag
   SETB  TR0                           ; 1 and restart it

   MOV   A,R4                          ; store the count in the buffer
   MOVX  [DPTR],A
   INC   DPTR
   MOV   A,R5
   MOVX  [DPTR],A
   INC   DPTR
   MOV   A,B
   MOVX  [DPTR],A
   INC   DPTR

*--- decide if we are done yet

   JB    F0FLAG,?RawEnd                ; force exit on timeout

   DecRR R3,R2
   JNZRR R3,R2,?RawLp

   ORL   ErrFlags,#$40                 ; record for debugging

*--- IR signal timed out or buffer full, so we are done

?RawEnd

   CLR   TR0                           ; restart normal timing
   MOV   TL0,T0Reload
   MOV   TH0,T0Reload+1
   CLR   TF0
   SETB  TR0

   POP   IE                            ; restore interrupts

   MOV   A,#LOW(MAXPULSES)             ; convert downcount to upcount
   CLR   C                             :  (need starting point)
   RLC   A
   MOV   R4,A
   MOV   A,#HIGH(MAXPULSES)
   RLC   A
   MOV   R5,A

   DecRR R3,R2                         ; account for first pulse...

   MOV   A,R4                          ; start - remaining count = upcount
   CLR   C
   SUBB  A,R2
   MOV   R2,A
   MOV   A,R5
   SUBB  A,R3
   MOV   R3,A

   MOV   A,R3                          ; convert to pulses from entries
   CLR   C
   RRC   A
   MOV   R3,A
   MOV   A,R2
   RRC   A
   MOV   R2,A

   MOV   A,ErrFlags                    ; save oddness flag
   MOV   A.4,C
   MOV   ErrFlags,A

   MOV   DPTR,#MCPulses                ; save the pulse counter
   MOV   A,R2
   MOVX  [DPTR],A
   INC   DPTR
   MOV   A,R3
   MOVX  [DPTR],A

?RawDone

}

   Nputstr("  Raw data captured!\n");

}


/*---------------------------------------------------------------------------*/
/* Capture MCIR signal and store in RAM data area                            */

MCCapture(RecID)
int RecID;
{
WORD MoveLen;

/*--- validate input, then find and remove existing record                   */
/*    the move assumes overlapping areas are moved bottom-to-top             */

   if ((!RecID) || (RecID > MAXRECNUM)) {
      sprintf(CmdRespBuff,"*** Record number must be between 1 and %u\n",
               MAXRECNUM);
      Nputstr(CmdRespBuff);
      goto Done;
   }

   pMCData = pMCRAM;                   /* data must be in RAM                */

   if (MCFetchRec(RecID) > 0) {
      MoveLen = MCRAMSize - (pMCNext - pMCRAM);
      if (REPORT_CAPTURE & ReportMode) {
         sprintf(CmdRespBuff,
                 "Removing record: pMCThis %04x pMCNext %04x Length %04x\n",
                 pMCThis,pMCNext,MoveLen);
         Nputstr(CmdRespBuff);
      }
      move(pMCThis,pMCNext,MoveLen);
      --(*(WORD *)pMCRAM);
      pMCNext = pMCThis;
   }

   if (REPORT_CAPTURE & ReportMode) {
   Nputstr("--- RAM buffer ready for new signal\n");
   MCDumpState();
   }

   if (MCFetchRec(MAXRECNUM) < 0) {    /* set pMCNext to terminators         */
      Nputstr("*** Storing first record in new structure\n");
      MCInitData();
   }

/*--- capture and convert a new signal                                       */

   Nputstr("--- Send signal from IR remote, press Enter to quit...\n");

   MCCapRaw();                   /* capture the raw IR signal & count        */

   if (!MCPulses) {
      Nputstr("No pulses stored, capture cancelled\n");
      goto Done;
   }

   if (REPORT_CAPTURE & ReportMode) {
   Nputstr("--- Timer buffer = cumulative times\n");
   MCDumpState();
   }

   MCCvtFromTimer();             /* get back to natural timing units         */

/*--- add new signal to end of existing data in RAM                          */

   MCRecID = RecID;              /* set up global variable for this          */
                                 /* MCPulses is already loaded               */

   if (REPORT_CAPTURE & ReportMode) {
   Nputstr("--- Timer buffer = incremental times\n");
   MCDumpState();
   }

   MCCompress(pMCNext,MCRAMSize-(int)(pMCNext-pMCRAM));

   if (REPORT_CAPTURE & ReportMode) {
   Nputstr("--- New signal stored in RAM\n");
   MCDumpState();
   }


   if (MCPulses <= 0) {
      Nputstr("*** Catured data not stored\n");
   }
   else {
      sprintf(CmdRespBuff,"There are %u bytes of RAM available\n",
              MCRAMSize - (pMCNext - pMCRAM));
      Nputstr(CmdRespBuff);
   }

Done:

}


/*---------------------------------------------------------------------------*/
/* Default IR signal, hand coded for debugging this thing...                 */
/* This should be overlaid with whatever real signals you need               */
/* Time intervals are measured in units of 8 ticks @ 1.085 us = 8.68 us      */
/*  10 ms = 1152 counts    1 ms = 115 counts                                 */
/*  52 us = 6 counts       95 us = 11 counts                                 */
/* Each pulse has an ON and OFF time interval, so # pulses = (# times)/2     */
/* Two-byte values are stored least-significant bit first just like ints     */
/* These are real times (in 8.7 us units) and get fudged to be correct later */

DummyData() {

#ifdef x0

 asm {
DDATA    EQU *
   ORG   $7800                   ; base of dummy data area

   DRW   2                       ; number of IR records

*--- first IR record

   DRW   100                     ; record ID number (not sequential)
   DRW   10                      ; total number of pulses in record

   DRW   1152                    ; ON time for first pulse
   DRW   576                     ; OFF time for first pulse

   DB    4                       ; number of pulses with 1-byte times

   DB    20                      ; ON time
   DB    30                      ; OFF time
   DB    40
   DB    50
   DB    60
   DB    70
   DB    80
   DB    90

   DB    4                       ; number of pulses with 2-byte times
   DRW   100                     ; ON time
   DRW   200                     ; OFF time
   DRW   300
   DRW   400
   DRW   500
   DRW   600
   DRW   700
   DRW   800

   DB    1                       ; number of pulses with 1-byte times
   DB    100                     ; ON time
   DB    200                     ; OFF time

   DB    0                       ; number of pulses with 2-byte times
   DB    0                       ; number of pulses with 1-byte times

*--- second IR record

   DRW   50                      ; record ID number (not sequential)
   DRW   2                       ; total number of pulses in record

   DRW   100                     ; ON time for first pulse
   DRW   200                     ; OFF time for first pulse

   DB    0                       ; number of pulses with 1-byte times

   DB    1                       ; number of pulses with 2-byte times
   DRW   1000                    ; ON time
   DRW   2000                    ; OFF time

   DB    0                       ; number of pulses with 1-byte times
   DB    0                       ; number of pulses with 2-byte times

*--- terminating marker after all records

   DB    0
   DB    0

   ORG   DDATA                   ; restore org for rest of code

 }

#endif

}
