/* REFRESH.C -- X10 message refreshing                                       */

#define NOREFRESHxxx 1                 /* define to eliminate refresh output */

/*---------------------------------------------------------------------------*/
/* Set up timers and suchlike                                                */

RefInitialize() {

   RefClear(REFPD_DEF);                /* clear the table & set new period   */

}


/*---------------------------------------------------------------------------*/
/* Return number of events in refresh table                                  */

int RefGetEvents() {

int NumEvents;
BYTE House;
BYTE Device;

   NumEvents = 0;

   for (House=0; House<16; ++House) {
      for (Device=0; Device<16; ++Device) {
         if (RefMsgs[House][Device]) {
            ++NumEvents;
         }
      }
   }

   return NumEvents;

}


/*---------------------------------------------------------------------------*/
/* Dump contents of refresh table                                            */

RefDumpTable() {

BYTE HIndex;
BYTE MIndex;
BYTE ShowHdr;

   for (HIndex=0; HIndex<16; ++HIndex) {
      ShowHdr =1;
      for (MIndex=0; MIndex<16; ++MIndex) {
         if (RefMsgs[HIndex][MIndex]) {
            if (ShowHdr) {
               sprintf(CmdRespBuff,"%c: ",HIndex+'A');
               Nputstr(CmdRespBuff);
               ShowHdr = 0;
            }
            sprintf(CmdRespBuff,"%u=",MIndex+1);
            Nputstr(CmdRespBuff);
            PwrShowDFName(RefMsgs[HIndex][MIndex] >> 8);
            Nputch(' ');
         }
      }
      if (!ShowHdr) {
         Nputch('\n');
      }
   }

}

/*---------------------------------------------------------------------------*/
/* Add an event to the refresh list                                          */
/* House, Device, Func are all indexes into our tables                       */
/* ON/OFF commands update the refresh status                                 */
/* BRIGHT/DIM command clear the status to prevent bad updates                */
/* ALLOFF command clears all devices for given house code                    */

RefNewEvent(House,Device,Func,Repeats)
WORD House;
WORD Device;
WORD Func;
WORD Repeats;
{
WORD Event;
BYTE Report;
char *pHdr;

   Report = (0 != (ReportMode & REPORT_REFRESH));

   switch (Func) {
   case FNDX_ON :
   case FNDX_OFF :
      Repeats = max(1,Repeats);        /* ensure non-zero repeat count!      */
      Event = ((Func & 0x001F) << 8) | Repeats;    /* set up entry           */

      if (RefMsgs[House][Device]) {    /* existing event?                    */
         if (RefMsgs[House][Device] == Event) {
            Report = 0;                /* yes, no change = no display        */
         }
         else {
            pHdr = "Chg Ref";
         }
      }
      else {
         pHdr = "Ref";
      }
      RefMsgs[House][Device] = Event;
      break;
   case FNDX_DIM :
   case FNDX_BRIGHT :
      if (RefMsgs[House][Device]) {    /* existing event?                    */
         RefMsgs[House][Device] = 0;               /* clear it               */
         pHdr = "Clr Ref";
      }
      else {
         Report = 0;
      }
      break;
   case FNDX_ALLOFF :
      for (Device=0; Device<16; ++Device) {
         if (RefMsgs[House][Device]) {
            RefMsgs[House][Device] = 0;            /* clear it               */
            if (Report) {
               sprintf(CmdRespBuff,"Clr All ref for H=%c\n",House+'A');
               Nputstr(CmdRespBuff);
               Report = 0;
            }
         }
      }
      Report = 0;
      break;
   default :
      Report = 0;
   }

   if (Report) {
      Nputstr(pHdr);
      Nputstr(":  H=");
      PwrShowHBits(House);
      Nputstr(" D=");
      PwrShowDFBits(Device);
      Nputstr(" F=");
      PwrShowDFBits(Func);
      sprintf(CmdRespBuff," Rep=%-2u (",Repeats);
      Nputstr(CmdRespBuff);
      PwrShowDFName(Device);
      Nputch(' ');
      PwrShowDFName(Func);
      Nputstr(")\n");
   }

}

/*---------------------------------------------------------------------------*/
/* Remove an event from the refresh list                                     */
/* House, Device are indexes into our tables                                 */

RefRemoveEvent(House,Device)
WORD House;
WORD Device;
{

   if (RefMsgs[House][Device]) {       /* make sure it exists...             */
      if (0 != (ReportMode & REPORT_REFRESH)) {
         sprintf(CmdRespBuff,"Del ref H=");
         Nputstr(CmdRespBuff);
         PwrShowHBits(House);
         Nputstr(" D=");
         PwrShowDFBits(Device);
         Nputch('\n');
      }
      RefMsgs[House][Device] = 0;      /*  before removing it                */
   }


}


/*---------------------------------------------------------------------------*/
/* Set refresh cycle and handle boundary cases                               */

RefSet(Period)
unsigned int Period;
{

   RefPeriod = Period;

   RefNext = 0;                        /* restart this from scratch          */
   RefHouse = RefDevice = 0;           /* starting from the top again        */

}

/*---------------------------------------------------------------------------*/
/* Set refresh period and clear the refresh table                            */

RefClear(Period)
unsigned int Period;
{

   RefSet(Period);

   memset(RefMsgs,0,sizeof(RefMsgs));  /* and no messages ready to go        */

}


/*---------------------------------------------------------------------------*/
/* Send the next refresh event...                                            */

RefDoRefresh() {

WORD Func;
unsigned int Reps;
unsigned int NumScanned;

   NumScanned = 0;

   do {                                            /* find next message      */

      RefDevice = 0x0F & ++RefDevice;              /* step to next device    */
      if (!RefDevice) {
         RefHouse = 0x0F & ++RefHouse;             /* then to next house     */
      }

      if (256 <= ++NumScanned) {                   /* bail out on no msgs    */
         break;
      }

   } while (!RefMsgs[RefHouse][RefDevice]);


   if (RefMsgs[RefHouse][RefDevice]) {             /* if good message        */
      Func = (RefMsgs[RefHouse][RefDevice] & 0x1F00) >> 8;
      Reps = RefMsgs[RefHouse][RefDevice]  & 0x00FF;
#ifndef NOREFRESH
      PwrSendSeq(RefHouse,RefDevice,Func,Reps);
#endif
   }

}


/*---------------------------------------------------------------------------*/
/* Decide whether it's time to send a refresh message...                     */

RefCheck() {

unsigned int IdealTime;
int NumEvents;

   ++RefNext;                          /* tick running timer                 */

   if (RefPeriod) {                    /* time to do it?                     */
      NumEvents = RefGetEvents();      /* anything to do?                    */
      if (NumEvents) {
         IdealTime = max(1,RefPeriod/NumEvents);            /* when?         */
         if (RefNext >= IdealTime) {                        /* now?          */
            RefDoRefresh();                                 /* do it!        */
            RefNext = 0;                                    /* retiming      */
         }
      }
   }
   else {
      RefNext = 0;                     /* nothing to do, reset timeout       */
   }


}


