/* Character generator routines for Vid-Link                                 */

#define CLEAR_NORMAL                   /* remove suffix to clear debug lines */

/*---------------------------------------------------------------------------*/
/* Put font bitmap at end of EPROM space                                     */
/* This is convenient for folks who want to use their own fonts...           */
/*  it starts at 1800 regardless of the rest of the code                     */
/* It must have 256 chars in 8x8 bitmap font, top first, left dot = bit 7    */

asm {

CGhere   EQU *
   ORG   $1800

FontTable
   INCLUDE "charfont.asm"

   ORG   CGhere

}


/*---------------------------------------------------------------------------*/
/* Character generator initialization                                        */

CGenInit(void) {

   CGCurrentRow  = CGCurrentCol  = 0;

asm {
   MOV   A,SwitchCopy                  /* get switch settings                */

   MOV   R0,#CG_OFF                    /* set up brightness                  */
   MOV   R1,#CG_NORMAL
   JNB   A.SW_STARTBRIGHT,CGI_1        /* use bright dots if switch is set   */
   MOV   R1,#CG_BRIGHT
CGI_1

   JNB   A.SW_STARTINV,CGI_2           /* flip if we want inverse chars      */
   XCH   A,R0                          /*  ... leaving A unchanged           */
   XCH   A,R1
   XCH   A,R0
CGI_2

   MOV   CGLowIntensity,R0
   MOV   CGHighIntensity,R1

   MOV   C,A.SW_STARTLARGE             /* figure out initial char size       */
   CLR   A
   RLC   A
   MOV   TestBits,A
}

   CGenSize(TestBits ? CG_LARGE : CG_SMALL);

}


/*---------------------------------------------------------------------------*/
/* Set character size                                                        */

CGenSize(CharSize)
BYTE CharSize;
{

   switch (CharSize) {
   case CG_SMALL :
   default:
      CLRBIT(B_CGDoubleSize);
      CGCharHeight = CG_FONTHEIGHT;
      CGCharWidth  = CG_FONTWIDTH;
      break;
   case CG_LARGE :
      SETBIT(B_CGDoubleSize);
      CGCharHeight = CG_FONTHEIGHT*2;
      CGCharWidth  = CG_FONTWIDTH*2;
   }

   CGNumCharRows = (MAX_ACTIVE_LINES - CG_TOPMARGIN - CG_BOTMARGIN) /
                    CGCharHeight;
   CGNumCharCols = (MAX_ACTIVE_DOTS - CG_LEFTMARGIN - CG_RIGHTMARGIN) /
                    CGCharWidth;

   CGCurrentRow = min(CGCurrentRow,CGNumCharRows-1);
   CGCurrentCol = min(CGCurrentCol,CGNumCharCols-1);

}


/*---------------------------------------------------------------------------*/
/* Write a character glyph into the buffer at a specific row and column      */
/* We disable video around this process because it is in the visible area    */
/* This does not update the current row or column values                     */

CGenWriteGlyph(CharCode,CharRow,CharCol)
BYTE CharCode;
BYTE CharRow;
BYTE CharCol;
{

asm {
   MOV   C,B_VidEnable                 /* save and turn off video enable     */
   MOV   F0FLAG,C
   CLR   B_VidEnable
   CLR   IO_VIDENABLE

   MOV   R0,#-9                        /* snag params from stack             */
   LCALL ?auto0                        /*  ... aim R0 at CharCode            */

   MOV   A,[R0]                        /* CharCode                           */
   MOV   B,#0
   MOV   R3,#3                         /*  ... times 8 = table offset        */
   LCALL ?shl
   ADD   A,#LOW FontTable
   XCH   A,B
   ADDC  A,#HIGH FontTable
   MOV   R5,A                          /* R5:R6 holds the font pointer       */
   MOV   R6,B

   INC   R0                            /* fetch the character row            */
   INC   R0
   MOV   A,[R0]                        /* CharRow                            */
   MOV   B,CGCharHeight                /*  ... times dot rows/char           */
   MUL   A                             /* A is line number in char area      */
   ADD   A,#CG_TOPMARGIN               /* plus top margin                    */
   MOV   P1,A                          /* P1 is video buffer high byte       */

   INC   R0                            /* fetch character column             */
   INC   R0
   MOV   A,[R0]                        /* CharCol                            */
   MOV   B,CGCharWidth                 /*  ... times dot cols/char           */
   MUL   A
   ADD   A,#CG_LEFTMARGIN              /* plus left margin                   */
   MOV   R7,A                          /* P1:R7 holds video buffer pointer   */

/*--- do an entire character from the font table                             */

   MOV   R2,#CG_FONTHEIGHT             /* we do a whole character here       */

?CWC_1

   MOV   DPH,R5                        /* fetch font byte                    */
   MOV   DPL,R6
   CLR   A
   MOVC  A,[A+DPTR]
   MOV   R0,A                          /* save the bits                      */
   MOV   R4,A
   INC   DPTR                          /* tick the font pointer (16 bits!)   */
   MOV   R5,DPH
   MOV   R6,DPL

   MOV   DPH,#HIGH VIDEO_BUS           /* select video buffer for writes     */

   JB    B_CGDoubleSize,?CWC_double

/*--- handle single-size chars                                               */

   MOV   R3,#CG_FONTWIDTH              /* set up for all eight bits          */
   MOV   DPL,R7                        /* starting at this address           */

?CWC_2
   MOV   A,R0                          /* pick up the font bits              */
   RLC   A
   MOV   R0,A                          /* save bits for next pass            */
   MOV   A,CGLowIntensity              /* figure out dot intensity           */
   JNC   ?CWC_Off1
   MOV   A,CGHighIntensity
?CWC_Off1

   MOVX  [DPTR],A                      /* write the dot                      */
   INC   DPL                           /* step to next dot                   */

   DJNZ  R3,?CWC_2

   INC   P1                            /* step to next buffer line           */

   DJNZ  R2,?CWC_1                     /* repeat for all font lines          */

   SJMP  ?CWC_done

/*--- handle double-size chars                                               */

?CWC_double

   MOV   R1,#2                         /* need two lines per font line       */

?CWC_twolines

   MOV   R3,#CG_FONTWIDTH              /* set up for all eight bits          */
   MOV   DPL,R7                        /* at this address                    */

?CWC_3
   MOV   A,R0                          /* pick up the font bits              */
   RLC   A
   MOV   R0,A                          /* save bits for next pass            */
   MOV   A,CGLowIntensity              /* figure out dot intensity           */
   JNC   ?CWC_Off2
   MOV   A,CGHighIntensity
?CWC_Off2

   MOVX  [DPTR],A                      /*  ... write it                      */
   INC   DPL
   MOVX  [DPTR],A                      /*  ... twice!                        */
   INC   DPL                           /* step to next dot                   */
   DJNZ  R3,?CWC_3

   INC   P1                            /* step to next buffer line           */
   MOV   ?R0,R4                        /* restore the font bits              */

   DJNZ  R1,?CWC_twolines              /* duplicate the line                 */

   DJNZ  R2,?CWC_1                     /* repeat for all font lines          */

?CWC_done

   MOV   P1,VidActiveLine              /* aim hardware at the right line     */
   MOV   C,F0FLAG                      /* restore previous video             */
   MOV   B_VidEnable,C
   MOV   IO_VIDENABLE,C

}

   CharCode = CharRow = CharCol = 0;

}


/*---------------------------------------------------------------------------*/
/* Clear the visible part of the buffer                                      */
/* This has a special-case hack to spare the bottom margin...                */
/*  so we can display test data in those lines & watch it chug along         */

CGenClear(void) {

asm {
   MOV   C,B_VidEnable                 /* save and turn off video enable     */
   MOV   F0FLAG,C
   CLR   B_VidEnable
   CLR   IO_VIDENABLE


   MOV   DPTR,#VIDEO_BUS               /* aim DPTR at the buffer             */
#ifdef CLEAR_NORMAL
   MOV   P1,#MAX_ACTIVE_LINES-1        /* start on last visible row          */
#else
   MOV   A,CGNumCharRows
   MOV   B,CGCharHeight
   MUL   A
   ADD   A,#CG_TOPMARGIN-1
   MOV   P1,A
#endif

   MOV   A,CGLowIntensity              /* this observes inverse setting!     */

CF_clear
   MOVX  [DPTR],A
   DJNZ  DPL,CF_clear                  /* clear a line                       */
   DJNZ  P1,CF_clear                   /* step up one line and go again      */

CF_clear0
   MOVX  [DPTR],A
   DJNZ  DPL,CF_clear0                 /* clear line zero                    */

   MOV   P1,VidActiveLine              /* aim hardware at the right line     */
   MOV   C,F0FLAG                      /* restore previous video             */
   MOV   B_VidEnable,C
   MOV   IO_VIDENABLE,C
}

   CGCurrentRow = CGCurrentCol = 0;    /* reset to the upper left corner     */

}

/*---------------------------------------------------------------------------*/
/* Place a character into the buffer                                         */
/* Pays attention to a minimal set of ASCII control codes                    */
/* Updates all the current locations and suchlike                            */

CGenPutChar(CharCode)
BYTE CharCode;
{
BYTE NumBlanks;

   switch (CharCode) {
   case LF :
      CGCurrentRow = ++CGCurrentRow % CGNumCharRows;
      break;

   case CR :
      CGCurrentCol = 0;                            /* go to leftmost column  */
      break;

   case BS:
      if (CGCurrentCol > 0) {                      /* back up if possible    */
         --CGCurrentCol;
         CGenPutChar(' ');
         --CGCurrentCol;
      }
      break;

   case FORMFEED :
      CGenClear();                                 /* flush the whole show   */
      break;

   case TAB :
      if ((CGCurrentCol + TAB_INTERVAL) < CGNumCharCols) {
         for (NumBlanks = TAB_INTERVAL - ((CGCurrentCol + 1) % TAB_INTERVAL);
              NumBlanks > 0;
              --NumBlanks) {
            CGenWriteGlyph(' ',CGCurrentRow,CGCurrentCol);
            ++CGCurrentCol;
         }
      }
      break;

   default :
      CGenWriteGlyph(CharCode,CGCurrentRow,CGCurrentCol);
      ++CGCurrentCol;
      if (CGCurrentCol >= CGNumCharCols) {
         CGCurrentCol = 0;
         TESTBIT(TestBits,B_LineWrap);
         if (!TestBits) {
            CGenPutChar(LF);
         }
      }
   }

}



