/* Video synchronizing routines for Vid-Link                                 */
/* Also includes the data data pumps for video buffer storage                */

#define PARTLINESxxx                   /* PARTLINES allows half-active lines */
#define TRACEOUTxxx                    /* TRACEOUT produces interrupt trace  */

/*---------------------------------------------------------------------------*/
/* CAUTION!                                                                  */
/* The video RAM is always enabled, so you MUST set P1 to a junk line BEFORE */
/*  writing to the 8254 timer ports...                                       */
/* That will confine invalid data to one known line                          */
/*  Otherwise, you will get spurious data on active video lines              */

/*---------------------------------------------------------------------------*/
/* Initial video sync interrupt entry point                                  */
/* This must end before the serial vector at 0023                            */
/* Thou mayest NOT have any local variables here... because:                 */
/*  the standard function does not have any prolog code, but adding          */
/*  local variables puts a stack adjustment before the first line of code    */

Int0Handler(void) {
   asm {

?Int0here   EQU *
      ORG   $0003                      * get to the external int vector addr

#ifdef TRACEOUT
      SETB  IO_TRANSDISABLE             **** mark our entry for debugging
#endif

      JNB   B_VidEnable,?Int0P1        * do not change P1 if video is OFF
      MOV   P1,VidActiveLine           * set high address byte for this line
?Int0P1
      INC   VidActiveLine              * always maintain this count!

      JNB   TF0,?Int0Bail              * bail out if timer is still running

      PUSH  PSW                        * save bystanders
      PUSH  A
      PUSH  DPL
      PUSH  DPH

      MOV   A,VidHandlerID             * dive into the current handler
      MOV   DPTR,#HandlerTable
      JMP   [A+DPTR]

?Int0Bail
#ifdef TRACEOUT
      CLR   IO_TRANSDISABLE
#endif
      RETI                             * otherwise, keep on trucking

      ORG   ?Int0here
   }
}


/*---------------------------------------------------------------------------*/
/* Video handler                                                             */
/* This gets control after the initial interrupt entry point                 */
/* Still no local variables, as we are in a hardware interrupt routine       */
/* CAUTION -- make sure the ORG is not lower than the current address!       */
/*  all AJMPs and targets must be in a single 2K page...                     */

VidIntHandlers(void) {

asm {
***** ORG   $1000                      * force to next page for AJMPs

HandlerTable
      AJMP  VidIdle                    *  0 idle line handler, nothing happens
      AJMP  VidVSync                   *  1 start vertical sync pulse
      AJMP  VidEqShort                 *  2 set equalizing pulses 1 count short
      AJMP  VidF1_Blank                *  3 start field 1 vertical blanking
      AJMP  VidF1_Active               *  4 last blank line atop field 1
      AJMP  VidF1_End                  *  5 last full line in field 1
      AJMP  VidF2_Blank                *  6 set up for last equalizing pulse
      AJMP  VidF2_Active               *  7 last blank line atop field 2
      AJMP  VidF2_End                  *  8 last active line in field 2
      AJMP  Vid_DataPump               *  9 data data pump

*--- common return point for all routines
*    we prepare for the next state, which takes effect when Timer 0 ticks

VidReturn
      INC   VidStateID                 * step to next state
      MOV   A,VidStateID               * to get handler ID and rep count
      MOV   DPTR,#VidStates            *  ... from state table
      MOVC  A,[A+DPTR]

      RL    A                          * convert to handler table index
      MOV   VidHandlerID,A

*--- return from this interrupt

      POP   DPH                        * restore bystanders
      POP   DPL
      POP   A
      POP   PSW

#ifdef TRACEOUT
      CLR   IO_TRANSDISABLE
#endif

      RETI                             * supplants the normal function RET

}

}

/*---------------------------------------------------------------------------*/
/* Video state handlers                                                      */
/* These are entered by the video interrupt handler...                       */
/* They are bottled up in one C routine to eliminate superfluous RETs        */

VidHandlers(void) {

asm {

*-- do nothing...

VidIdle
      AJMP VidReturn

*--- set vertical sync interval
*    previous equalizing pulses set up the period & blanking
*    displays data during last equ pulse before vertical sync

VidVSync

      LoadT0   5*T0_HALFLINE-T0_QUARTERLINE        * set delay to 2nd eqs

      PUSH  P1                         * set P1 to junk line
      MOV   P1,#VID_JUNKLINE

      WaitON   IE0                     * capture last equalizing pulse

      SETB  TR0                        * start delay timer

      MOV   DPTR,#I8254_DURATION       * set up vertical sync width
      MOV   A,#SYNC_VSYNC
      MOVX  [DPTR],A

      POP   P1

      AJMP VidReturn

*--- set equalizing pulses that are short by one count

VidEqShort
      LoadT0   4*T0_HALFLINE-T0_QUARTERLINE        * delay to next-to-last eq

      PUSH  P1                         * set P1 to junk line
      MOV   P1,#VID_JUNKLINE

      WaitON   IE0                     * capture last Vsync pulse

      SETB  TR0                        * start delay timer

      MOV   DPTR,#I8254_PERIOD         * set up short half-line period
      MOV   A,#PERIOD_HALFSHORT
      MOVX  [DPTR],A

      MOV   DPTR,#I8254_DURATION       * set up equalizing pulses
      MOV   A,#SYNC_EQUALIZING
      MOVX  [DPTR],A

      POP   P1

      AJMP VidReturn

*--- set vertical blanking pulses for field 1
*    this is slightly different in Field 1 and Field 2...

VidF1_Blank
      LoadT0   2*T0_NORMLINE-T0_QUARTERLINE        * set delay to data pump

      WaitON   IE0                     * capture next-to-last equalizing pulse

      PUSH  P1                         * set P1 to junk line
      MOV   P1,#VID_JUNKLINE

      WaitON   IE0                     * capture last equalizing pulse

      SETB  TR0                        * start delay timer

      MOV   DPTR,#I8254_PERIOD         * set up normal line period
      MOV   A,#PERIOD_NORMAL
      MOVX  [DPTR],A

      INC   DPTR                       * set up normal sync pulse duration
      MOV   A,#SYNC_NORMAL
      MOVX  [DPTR],A

      INC   DPTR                       * set up complete blanking for line
      MOV   A,#BLANK_COMPLETE
      MOVX  [DPTR],A

      MOV   VidActiveLine,#MAX_ACTIVE_LINES        * ensure no wrap

      POP   P1

      AJMP VidReturn

*--- set normal blanking and enable video at top of active area
*    the lines are fully blanked, so IO_VIDENABLE works on the next line
*    the active portion is delayed to give the data pump more time

VidF1_Active
      LoadT0   T0_ACTIVE-(4*T0_NORMLINE)-T0_QUARTERLINE

      PUSH  P1                         * set P1 to junk line
      MOV   P1,#VID_JUNKLINE

      WaitON   IE0                     * capture last blanking pulse

      SETB  TR0                        * start delay timer

      MOV   DPTR,#I8254_BLANKING       * set up normal blanking duration
      MOV   A,#BLANK_NORMAL
      MOVX  [DPTR],A

      MOV   VidActiveLine,#4           * set up for top line of video
      MOV   C,B_VidEnable              * enable if desired
      MOV   IO_VIDENABLE,C

      POP   P1

      AJMP VidReturn


*--- set special half line at bottom of first field
*    we capture that sync and two equalizing pulses to get good timing
*    timing is tight for the last full active line

VidF1_End

      PUSH  P1                         * save P1 around junk line

      MOV   DPTR,#I8254_PERIOD         * set up half-line period
      MOV   A,#PERIOD_HALFSHORT-1
?L1_End0
      JNB   IE0,?L1_End0
      MOV   P1,#VID_JUNKLINE
      MOVX  [DPTR],A
      POP   P1

      JNB   B_VidEnable,?L1_End1
      MOV   P1,VidActiveLine           * display half line of video
?L1_End1
      INC   VidActiveLine

      CLR   IE0

      WaitON   IE0                     * catch last active line sync
      JNB   B_VidEnable,?L1_End2
      MOV   P1,VidActiveLine           * and display the right stuff!
?L1_End2

#ifdef PARTLINES
      MOV   A,#5
?wait_mid261
      DJNZ  A,?wait_mid261
#endif

      CLR   IO_VIDENABLE               * all video is blanked now

      PUSH  P1                         * set P1 to junk line
      MOV   P1,#VID_JUNKLINE

      MOV   DPTR,#I8254_DURATION       * set up equalizing sync duration
      MOV   A,#SYNC_EQUALIZING
      MOVX  [DPTR],A

      INC   DPTR                       * set up complete blanking
      MOV   A,#BLANK_COMPLETE
      MOVX  [DPTR],A

      WaitON   IE0                     * catch first equalizing sync

      SETB     B_Field2                * we are now in Field 2

      LoadT0   T0_NORMLINE+T0_QUARTERLINE          * set up delay to Vsync

      WaitON   IE0                     * catch second equalizing sync

      SETB  TR0                        * start delay until vertical sync

      MOV   DPTR,#I8254_PERIOD         * set up long half-line period
      MOV   A,#PERIOD_HALFLONG
      MOVX  [DPTR],A

      INC   VidFieldCtr                * account for this field

      POP   P1

      AJMP VidReturn

*--- handle transition to second field vertical blanking

VidF2_Blank
      LoadT0   3*T0_NORMLINE-T0_QUARTERLINE        * set delay to data pump

      PUSH  P1                         * set P1 to junk line
      MOV   P1,#VID_JUNKLINE

      WaitON   IE0                     * capture next-to-last equalizing

      MOV   DPTR,#I8254_PERIOD         * first line is short by one tick
      MOV   A,#PERIOD_NORMAL-1
      MOVX  [DPTR],A

      WaitON   IE0                     * capture last equalizing pulse

      SETB  TR0                        * start delay timer

      MOV   A,#PERIOD_NORMAL           * restore normal line duration
      MOVX  [DPTR],A

      INC   DPTR                       * set up normal sync pulse duration
      MOV   A,#SYNC_NORMAL
      MOVX  [DPTR],A

      MOV   VidActiveLine,#MAX_ACTIVE_LINES        * ensure no wrap

      POP   P1

      AJMP VidReturn

*--- set up for half-active line atop second field
*    they are delayed four lines to allow time for the data pump

VidF2_Active
      LoadT0   T0_ACTIVE-(4*T0_NORMLINE)-T0_QUARTERLINE

      PUSH  P1                         * set P1 to junk line
      MOV   P1,#VID_JUNKLINE

      WaitON   IE0                     * capture last blanking pulse

      SETB  TR0                        * start delay timer

      MOV   DPTR,#I8254_BLANKING       * unblank next line
      MOV   A,#BLANK_NORMAL
      MOVX  [DPTR],A

      POP   P1

#ifdef PARTLINES
      WaitON   IE0                     * capture first active line sync

      JNB   B_VidEnable,?F2_21P1
      MOV   P1,#4                      * set video address
?F2_21P1
      MOV   VidActiveLine,#5           * set up for next line

VidF2_21
      MOV   A,#13
?wait_mid21
      DJNZ  A,?wait_mid21              * stall to middle of line

      MOV   C,B_VidEnable              * enable if desired
      MOV   IO_VIDENABLE,C
#else

      MOV   C,B_VidEnable              * enable on next line if desired
      MOV   IO_VIDENABLE,C

      MOV   VidActiveLine,#4           * set up video address

#endif

      AJMP VidReturn

*--- set up equalizing pulses at bottom of second field
*    this also resets the overall sync state machine
*    we capture and blank the last active line
*     thus avoiding glitches when writing the timers for the halflines

VidF2_End

      WAITON IE0                       * capture last active line sync

      CLR   IO_VIDENABLE               * avoid scrambling the video

      PUSH  P1                         * set P1 to junk line
      MOV   P1,#VID_JUNKLINE

      MOV   DPTR,#I8254_PERIOD         * set up half-line period
      MOV   A,#PERIOD_HALFSHORT
      MOVX  [DPTR],A

      INC   DPTR                       * set up equalizing duration
      MOV   A,#SYNC_EQUALIZING
      MOVX  [DPTR],A

      INC   DPTR                       * set up complete blanking
      MOV   A,#BLANK_COMPLETE
      MOVX  [DPTR],A

***   MOV   DPTR,#SWITCHES             * trigger scope sync
***   MOVX  A,[DPTR]

      WAITON IE0                       * capture first equalizing pulse

      CLR   IO_VIDENABLE               * disable video on this line

      MOV   VidStateID,#VS_VSYNC-1     * reset state machine

      LoadT0   T0_NORMLINE+T0_QUARTERLINE          * set up delay to Vsync

      SETB     B_Field2                * we are now in Field 2

      WaitON   IE0                     * capture second equalizing pulse

      SETB  TR0                        * start delay until vertical sync

      MOV   DPTR,#I8254_PERIOD         * set up long half-line period
      MOV   A,#PERIOD_HALFLONG
      MOVX  [DPTR],A

      INC   VidFieldCtr                * account for this field

      POP   P1

      AJMP VidReturn

*--- pump data between Internal RAM buffer and video buffer
*    this occupies the vertical blanking intervals of both fields
*    does =not= change any sync timings or video blanking
*    we need not catch any syncs after the first because...
*     Timer 0 alerts the default handler at the right moment!

Vid_DataPump

      LoadT0   10*T0_NORMLINE-T0_QUARTERLINE       * set delay to active lines

      PUSH  ?R0                        * save bystanders
      PUSH  ?R1
      PUSH  B
      PUSH  P1

      MOV   A,PumpState                * decode pump state
      MOV   DPTR,#PumpJumps

      WAITON IE0
      SETB  TR0                        * begin delay until active lines

      JMP   [A+DPTR]

PumpJumps
      AJMP  VDP_idle
      AJMP  VDP_fromVRAM
      AJMP  VDP_toVRAM
      AJMP  VDP_initline
      AJMP  VDP_restartline

VDP_idle
      AJMP  VDP_done

*--- Move data from Video RAM to the IRAM FIFO
*    continues until VRAM is empty, then switches to IDLE

VDP_fromVRAM

      MOV   A,PumpFIFOCtr              * room for more?
      CJNE  A,#PUMPFIFOSIZE,VDPf_room
      SJMP  VDPf_done

VDPF_room
      MOV   P1,PumpLine                * set up video buffer line address
      MOV   DPTR,#VIDEO_BUS+PB_REMAINLOC           * fetch remaining count
      MOVX  A,[DPTR]

      JNZ   VDPf_go                    * anything left to do?

      MOV   PumpState,#PUMP_IDLE       * no, indicate we are done
      SJMP  VDPf_done

VDPf_go
      MOV   R1,A                       * set up max counter

      MOV   DPL,#PB_INDEXLOC           * fetch current index
      MOVX  A,[DPTR]
      MOV   DPL,A                      * DPTR -> char in video buffer

      MOV   R0,PumpFIFOHead            * R0 -> head of IRAM FIFO

VDPf_move
      MOVX  A,[DPTR]                   * fetch the byte from VRAM
      INC   DPL
      MOV   [R0],A                     * stash in the IRAM FIFO
      INC   R0
      CJNE  R0,#PumpFIFO+PUMPFIFOSIZE,VDPf_move1
      MOV   R0,#PumpFIFO
VDPf_move1
      DEC   R1                         * tick remaining count
      INC   PumpFIFOCtr                * tick FIFO count and stop when full
      MOV   A,PumpFIFOCtr
      CJNE  A,#PUMPFIFOSIZE,VDPf_move2
      SJMP  VDPf_move3
VDPf_move2
      CJNE  R1,#0,VDPf_move            * stop if no more chars remain

VDPf_move3
      MOV   PumpFIFOHead,R0            * save FIFO pointer

      MOV   A,DPL                      * save char index
      MOV   DPL,#PB_INDEXLOC
      MOVX  [DPTR],A

      MOV   DPL,#PB_REMAINLOC          * save remaining count
      MOV   A,R1
      MOVX  [DPTR],A

VDPf_done
      AJMP  VDP_done

*--- Move data to Video RAM from the IRAM FIFO
*    continues until VRAM is full, then replaces last one repeatedly

VDP_toVRAM

      MOV   A,PumpFIFOCtr              * anything to move?
      JZ    VDPt_done                  *  no, bail out with no change

      MOV   P1,PumpLine                * set up video buffer line address
      MOV   DPTR,#VIDEO_BUS+PB_REMAINLOC           * fetch char counter
      MOVX  A,[DPTR]

      CJNE  A,#PB_LINESIZE,VDPt_go     * room for any more chars?

      DEC   A                          * no, overwrite previous last char

      MOV   DPL,#PB_INDEXLOC           * force index to last char slot
      MOVX  [DPTR],A                   *  which equals count value...

VDPt_go
      MOV   R1,A                       * set up buffer contents counter

      MOV   DPL,#PB_INDEXLOC           * fetch current index
      MOVX  A,[DPTR]
      MOV   DPL,A                      * DPTR -> char in video buffer

      MOV   R0,PumpFIFOTail            * R0 -> tail of IRAM FIFO

VDPt_move
      MOV   A,[R0]                     * fetch FIFO byte
      INC   R0
      MOVX  [DPTR],A                   * stash in VRAM
      INC   DPL
      CJNE  R0,#PumpFIFO+PUMPFIFOSIZE,VDPt_move1
      MOV   R0,#PumpFIFO
VDPt_move1
      INC   R1                         * tick VRAM char count
      DJNZ  PumpFIFOCtr,VDPt_move2     * tick FIFO count and stop when empty
      SJMP  VDPt_move3
VDPt_move2
      CJNE  R1,#PB_LINESIZE,VDPt_move  * stop if no more room left

VDPt_move3
      MOV   PumpFIFOTail,R0            * save FIFO pointer

      MOV   A,DPL
      MOV   DPL,#PB_INDEXLOC
      MOVX  [DPTR],A

      MOV   DPL,#PB_REMAINLOC          * save count of chars in VRAM buffer
      MOV   A,R1
      MOVX  [DPTR],A

VDPt_done
      AJMP  VDP_done

*--- Initialize a VRAM buffer data storage line
*    we do =not= clear the line, as that takes FAR too long...

VDP_initline

      MOV   P1,PumpLine                * set up video buffer line address

      MOV   DPTR,#VIDEO_BUS+PB_REMAINLOC           * zero the counter
      CLR   A
      MOVX  [DPTR],A

      MOV   DPL,#PB_INDEXLOC           * zero the index
      MOVX  [DPTR],A

      MOV   PumpState,#PUMP_IDLE       * indicate we are done
      AJMP  VDP_done

*--- Restart a VRAM buffer data storage line without affecting the counter

VDP_restartline

      MOV   P1,PumpLine                * set up video buffer line address

      MOV   DPTR,#VIDEO_BUS+PB_INDEXLOC            * zero the index
      CLR   A
      MOVX  [DPTR],A

      MOV   PumpState,#PUMP_IDLE       * indicate we are done
      AJMP  VDP_done

/* Common interrupt exit point                                               */

VDP_done
      POP   P1                         * restore bystanders
      POP   B
      POP   ?R1
      POP   ?R0


      AJMP VidReturn

}
}

/*---------------------------------------------------------------------------*/
/* Initialize a video data pump buffer line                                  */
/* This requires at most one field time, as it runs during vertical blanking */

VidPumpInitLine(LineID,Function)
BYTE LineID;
BYTE Function;
{

   PumpLine = LineID;
   PumpState = Function;
   while (PUMP_IDLE != PumpState) {
      continue;
   };

}

